# -*- coding: utf-8 -*-
import os
from PyQt4 import QtCore, QtGui
import numpy, scipy.signal
import time, datetime
import copy
import pywt, pyqtgraph
import Queue

class PSDWorker(QtCore.QObject):
    # finished = QtCore.pyqtSignal(numpy.float64, numpy.float64)
    finished = QtCore.pyqtSignal()
    # PSDReady = QtCore.pyqtSignal(numpy.ndarray, numpy.ndarray)
    PSDReady = QtCore.pyqtSignal()
    histogramReady = QtCore.pyqtSignal()
    #Signal for progress bar
    progress = QtCore.pyqtSignal(int, str)

    def __init__(self, parentWindow, dictOfConstants):
        super(PSDWorker, self).__init__()
        # self.ADCData = numpy.zeros(dictOfConstants['FRAMELENGTH']/4)
        self.parentWindow = parentWindow
        self.dictOfConstants = dictOfConstants

    def calculatePSD(self):
        self.start = time.clock()
        # self.ADCData = window.ADCData
        self.ADCData[self.ADCData >= 2**(self.dictOfConstants['ADCBITS']-1)] -= 2**(self.dictOfConstants['ADCBITS'])
        self.ADCData /= 2**(self.dictOfConstants['ADCBITS']-1)
        self.ADCData -= numpy.mean(self.ADCData)
        # if (True == self.parentWindow.ui.checkBox_enableSquareWave.isChecked()):
            # self.calculateDCResistance()
        # if (window.columnSelect in [0, 3]):
            # self.ADCData *= 16.0/5
        self.parentWindow.histogramView, self.parentWindow.bins = numpy.histogram(self.ADCData, bins=64)
        self.parentWindow.bins /= (self.dictOfConstants['AAFILTERGAIN']*self.parentWindow.RDCFB)
        self.histogramReady.emit()
        self.progress.emit(2, 'Calculating PSD')
        #### Begin PSD calculation
        # f, Pxx = scipy.signal.periodogram(self.ADCData, dictOfConstants['ADCSAMPLINGRATE'], nfft=2**19)
        f, Pxx = scipy.signal.welch(self.ADCData, self.dictOfConstants['ADCSAMPLINGRATE'], nperseg=2**19)
        #### End PSD calculation

        #### Begin FFT calculation
        # NFFT = 2**22
        # ADCDataFFT = numpy.fft.fft(self.ADCData, NFFT)
        # ADCDataFFT = ADCDataFFT[1:NFFT/2]
        # f = ADCSAMPLINGRATE/2*numpy.linspace(0, 1, NFFT/2)
        # f = f[1:len(f)]
        # Pxx = 2.0/(ADCSAMPLINGRATE*NFFT)*numpy.abs(numpy.square(ADCDataFFT))
        #### End FFT calculation

        # f_100Hz = numpy.argmin(numpy.abs(f - 1e2))
        Pxx = Pxx[f > 1e2]
        f = f[f>1e2]
        # PxxFitCoefficients = numpy.polynomial.polynomial.polyfit(f[f<6e6], Pxx[f<6e6]*f[f<6e6], 3)
        # f, Pxx = f[f_100Hz:len(f)], Pxx[f_100Hz:len(Pxx)]
        self.progress.emit(2, 'Preparing PSD data for plotting')
        f_100kHz = numpy.argmin(numpy.abs(f - 1e5))
        f_1MHz = numpy.argmin(numpy.abs(f - 1e6))
        f_10MHz = numpy.argmin(numpy.abs(f - 10e6))
        logIndices = numpy.unique(numpy.asarray(numpy.logspace(0, numpy.log10(f_10MHz), f_10MHz/self.dictOfConstants['SUBSAMPLINGFACTOR']), dtype=numpy.int32))
        # fToDisplay = f[logIndices-1]
        # PxxToDisplay = Pxx[logIndices-1]
        self.parentWindow.f = f[logIndices-1]
        # print len(PxxFitCoefficients)
        # self.parentWindow.PSDFit = numpy.divide(numpy.polynomial.polynomial.polyval(f[logIndices-1], PxxFitCoefficients), f[logIndices-1])
        # print PxxFitCoefficients[2]/(4*3.14**2*3e-9*3e-9)
        if (hasattr(self.parentWindow.ui, 'checkBox_frequencyResponse') and self.parentWindow.ui.checkBox_frequencyResponse.isChecked() == True):
            self.parentWindow.PSD = numpy.divide(Pxx[logIndices-1], f[logIndices-1]**2)
        else:
            self.parentWindow.PSD = Pxx[logIndices-1]
        # PxxFitCoefficients = numpy.polynomial.polynomial.polyfit(self.parentWindow.f[self.parentWindow.f < 6e6], self.parentWindow.PSD[self.parentWindow.f < 6e6] * self.parentWindow.f[self.parentWindow.f < 6e6], 3, w = 1/(self.parentWindow.f[self.parentWindow.f < 6e6] * self.parentWindow.PSD[self.parentWindow.f < 6e6]))
        # self.parentWindow.PSDFit = numpy.divide(numpy.polynomial.polynomial.polyval(self.parentWindow.f, PxxFitCoefficients), self.parentWindow.f)
        if (self.parentWindow.ui.action_addNoiseFit.isChecked() == True):
            self.parentWindow.PSDFit = self.createFit(self.parentWindow.f, self.parentWindow.PSD, 1e6)
        # self.PSDReady.emit(fToDisplay, PxxToDisplay)
        self.PSDReady.emit()
        self.progress.emit(1, 'Finishing up')

        self.RMSNoise = numpy.sqrt(scipy.integrate.cumtrapz(Pxx, f, initial=0))/self.dictOfConstants['AAFILTERGAIN']/self.parentWindow.RDCFB
        self.parentWindow.RMSNoise = self.RMSNoise[logIndices-1]
        
        self.parentWindow.RMSNoise_100kHz = numpy.round(self.RMSNoise[f_100kHz] * 10**12, 1)
        self.parentWindow.RMSNoise_1MHz = numpy.round(self.RMSNoise[f_1MHz] * 10**12, 1)
        self.parentWindow.RMSNoise_10MHz = numpy.round(self.RMSNoise[f_10MHz-1] * 10**12, 1)
        self.stop = time.clock()
        # print "Calculate PSD thread took", self.stop-self.start, "s"
        # self.finished.emit(numpy.round(RMSNoise_100kHz * 10**12, 1), numpy.round(RMSNoise_1MHz * 10**12, 1))
        self.finished.emit()

    def calculateDCResistance(self):
        self.ADCDataRMS = numpy.sqrt(numpy.mean(numpy.power(self.ADCData, 2)))
        self.poreResistance = self.dictOfConstants['SQUAREWAVEAMPLITUDE']/self.ADCDataRMS*self.dictOfConstants['AAFILTERGAIN']*self.parentWindow.RDCFB
        self.parentWindow.ui.label_poreResistance.setText(str(numpy.round(self.poreResistance/1e6, 1)) + u"MΩ")
        
    def createFit(self, fFit, PSDFit, maxFitFrequency = 6e6):
        fFitNew = fFit[fFit < maxFitFrequency]
        PSDFitNew = PSDFit[fFit < maxFitFrequency]
        fitCoefficients = numpy.polynomial.polynomial.polyfit(fFitNew, PSDFitNew * fFitNew, 3, w = 1/(fFitNew * PSDFitNew)) #Fit PSD*f to 3rd order and then divide by f to get the 1/f term also
        print numpy.sqrt(fitCoefficients[3])/(2*3.14*3e-9*self.parentWindow.RDCFB)
        return numpy.divide(numpy.polynomial.polynomial.polyval(fFit, fitCoefficients), fFit)

class GetDataFromFPGAWorker(QtCore.QObject):
    """Worker module that inherits from QObject. Eventually, this gets moved on to a thread that inherits from QThread."""
    finished = QtCore.pyqtSignal()
    dataReady = QtCore.pyqtSignal()

    def __init__(self, parentWindow, dictOfConstants, FPGAInstance):
        super(GetDataFromFPGAWorker, self).__init__()
        self.parentWindow = parentWindow
        self.FPGAInstance = FPGAInstance
        if self.FPGAInstance is not None:
            self.FPGAType = self.FPGAInstance.type
            self.validColumns = self.FPGAInstance.validColumns
            self.processRawDataThreadOptions = {'Master': self.parentWindow.processRawDataMasterThread, 'Slave': self.parentWindow.processRawDataSlaveThread}
            self.processRawDataThread = self.processRawDataThreadOptions[self.FPGAType]
            self.processRawDataWorkerInstanceOptions = {'Master': self.parentWindow.processRawDataMasterWorkerInstance, 'Slave': self.parentWindow.processRawDataSlaveWorkerInstance}
            self.processRawDataWorkerInstance = self.processRawDataWorkerInstanceOptions[self.FPGAType]
        self.dictOfConstants = dictOfConstants

    def getDataFromFPGA(self):
        """Gets data from the FPGA and logs it if the corresponding option has been enabled. Calls processRawData once a chunk of data has been obtained. Loops infinitely."""
        while (self.FPGAInstance.configured):
            # rawData = bytearray(self.dictOfConstants['FRAMELENGTH'])
            if ('Master' == self.FPGAType):
                rawData = bytearray(self.dictOfConstants['FRAMELENGTH_MASTER'])
            elif ('Slave' == self.FPGAType):
                rawData = bytearray(self.dictOfConstants['FRAMELENGTH_SLAVE'])
            self.start = time.clock()
            errorReturn = self.FPGAInstance.xem.ReadFromBlockPipeOut(0xA0, self.dictOfConstants['BLOCKLENGTH'], rawData)
            # errorReturn = self.FPGAInstance.xem.ReadFromPipeOut(0xA0, rawData)
            if (self.FPGAInstance.xem.Timeout == errorReturn):
                print "Transaction timed out on", self.FPGAType, "FPGA"
            if (self.FPGAInstance.xem.Failed == errorReturn):
                print "Transaction failed on", self.FPGAType, "FPGA"
                self.FPGAInstance.powered = False #Change power status to off if code breaks out of the while loop
                self.FPGAInstance.configured= False #Change configuration status to unconfigured if code breaks out of the while loop
            if (self.parentWindow.columnSelect in self.validColumns):
                if (self.parentWindow.ui.action_enableLogging.isChecked()):
                    self.parentWindow.writeToLogFileWorkerInstance.rawData.put(rawData)
                    # self.parentWindow.writeToLogFileThread.start()
            self.dataReady.emit()
            # rawDataUnpacked = numpy.frombuffer(rawData, dtype='uint32')
            if (self.parentWindow.columnSelect in self.validColumns):
                if not (self.processRawDataThread.isRunning()):
                    self.processRawDataWorkerInstance.rawData = rawData
                    self.processRawDataThread.start()
                # if (False == window.processRawDataThread.isRunning()):
                    # window.processRawDataWorkerInstance.rawData = rawData
                    # window.processRawDataThread.start()
                self.updateWireOuts()
            self.stop = time.clock()
            # if ('Slave' == self.FPGAType):
                # print "Get data from FPGA thread took", self.stop-self.start, "s"
            # self.dataReady.emit()
            # self.processRawData(rawDataUnpacked)
        self.finished.emit()

    def updateWireOuts(self):
        self.FPGAInstance.xem.UpdateWireOuts()
        RAMMemoryUsage = self.FPGAInstance.xem.GetWireOutValue(0x20)
        RAMMemoryUsage = numpy.round(RAMMemoryUsage*1.0/2**20, 1);
        self.parentWindow.RAMMemoryUsage = RAMMemoryUsage
        return

class ProcessRawDataWorker(QtCore.QObject):
    finished = QtCore.pyqtSignal()
    # dataReady = QtCore.pyqtSignal(numpy.ndarray)
    dataReady = QtCore.pyqtSignal()
    startPSDThread = QtCore.pyqtSignal()
    #Signals for the progress bar
    progress = QtCore.pyqtSignal(int, str)

    def __init__(self, parentWindow, dictOfConstants, FPGAInstance):
        super(ProcessRawDataWorker, self).__init__()
        self.dictOfConstants = dictOfConstants
        self.createFilter(100e3)
        self.parentWindow = parentWindow
        self.FPGAInstance = FPGAInstance
        if self.FPGAInstance is not None:
            self.FPGAType = self.FPGAInstance.type
            self.validColumns = self.FPGAInstance.validColumns
        else:
            self.FPGAType = 'Generic'
            self.validColumns = [0, 1, 2, 3, 4]
        # self.rawData = bytearray(self.dictOfConstants['FRAMELENGTH'])
        self.rawDataUnpacked = 0

    def processRawData(self):
        """Processes the raw data. The raw data is a 32 bit number that contains {8'h00, 12'hADC1Data, 12'hADC0DATA}. This method unpacks the data and then subsamples it and corrects for the boosting and anti-aliasing filter gain and DC offset."""
        self.start = time.clock()
        # self.rawDataUnpacked = numpy.frombuffer(self.rawData, dtype='uint32')
        # ADCData = {0: numpy.bitwise_and(self.rawDataUnpacked, 0xfff),\
                # 1: (numpy.bitwise_and(self.rawDataUnpacked, 0xfff000))>>12,\
                # 3: numpy.bitwise_and(self.rawDataUnpacked, 0xfff)}
        # Switched from dictionary to if else implementation because of timing issues while displaying data
        ADCData = self.unpackData()
        if (self.parentWindow.ui.checkBox_enableLivePreviewFilter.isChecked()):
            # ADCData2 = copy.deepcopy(ADCData)
            ADCData2 = ADCData.astype(numpy.float64)
            ADCData2[ADCData2 >= 2**(self.dictOfConstants['ADCBITS']-1)] -= 2**(self.dictOfConstants['ADCBITS'])
            # ADCData2 = ADCData2.astype(numpy.float64, copy=False)
            ADCData2 /= 2**(self.dictOfConstants['ADCBITS']-1)
            # self.parentWindow.IDCRelative = numpy.mean(ADCData2) - self.parentWindow.IDCOffset
            # self.parentWindow.updateIDCLabels()
            ADCData2 = scipy.signal.lfilter(self.b, self.a, ADCData2)
            # ADCData = ADCData2
            # if (self.parentWindow.ui.checkBox_enableWaveletDenoising.isChecked()):
                # # threshold = 0.005# numpy.sqrt(2*numpy.log2(len(ADCData2)))
                # threshold = numpy.std(ADCData2)*numpy.sqrt(2*numpy.log10(len(ADCData2)))
                # self.motherWavelet = str(self.parentWindow.ui.lineEdit_motherWavelet.text())
                # waveletCoefficients = pywt.wavedec(ADCData2, self.motherWavelet, level=3)# level=int(numpy.floor(numpy.log2(len(ADCData2)))))
                # newWaveletCoefficients = map(lambda x: pywt.thresholding.soft(x, 0.5*threshold), waveletCoefficients)
                # ADCData2 = pywt.waverec(newWaveletCoefficients, self.motherWavelet)
            # totalNoise = numpy.std(ADCData2)
            # tags = ADCData2 < -5*totalNoise
            # print totalNoise/self.parentWindow.RDCFB, numpy.sum(numpy.bitwise_and(numpy.bitwise_and(tags[:-3], tags[1:-2]), numpy.bitwise_and(tags[2:-1], tags[3:])))

            # Skip first few points of filtered data to avoid edge effects due to filtering
            self.skipPoints = numpy.ceil(self.dictOfConstants['ADCSAMPLINGRATE']/self.livePreviewFilterBandwidth)
            dataToDisplay = (ADCData2)[self.skipPoints::self.dictOfConstants['SUBSAMPLINGFACTOR']]*1 #Multiplying by 1 is necessary to ensure dataToDisplay does not modify ADCData2
            if (hasattr(self.parentWindow, 'analyzeDataWorkerInstance')):
                self.parentWindow.analyzeDataWorkerInstance.rawData = ADCData2[self.skipPoints:]/self.parentWindow.RDCFB/self.dictOfConstants['AAFILTERGAIN'] - self.parentWindow.IDCOffset
                self.parentWindow.analyzeDataWorkerInstance.skipPoints = self.skipPoints
        else:
            dataToDisplay = (ADCData)[0::self.dictOfConstants['SUBSAMPLINGFACTOR']]*1.0
            dataToDisplay[dataToDisplay >= 2**(self.dictOfConstants['ADCBITS']-1)] -= 2**(self.dictOfConstants['ADCBITS'])
            dataToDisplay /= 2**(self.dictOfConstants['ADCBITS']-1)
            # self.parentWindow.IDCRelative = numpy.mean(dataToDisplay) - self.parentWindow.IDCOffset
            # self.parentWindow.updateIDCLabels()
            # dataToDisplay -= numpy.mean(dataToDisplay)
        # if (window.columnSelect in [0, 3]):
            # dataToDisplay *= 16.0/5
        dataToDisplay *= 1.0/self.parentWindow.RDCFB/self.dictOfConstants['AAFILTERGAIN'] #Prefixes (like nano or pico) are handled automatically by PyQtGraph
        self.parentWindow.IDCRelative = numpy.mean(dataToDisplay) - self.parentWindow.IDCOffset
        dataToDisplay -= self.parentWindow.IDCOffset
        self.parentWindow.updateIDCLabels()
        # if (self.parentWindow.ui.checkBox_enableLivePreviewFilter.isChecked()):
            # dataToDisplay = scipy.signal.lfilter(self.b, self.a, dataToDisplay)
        self.progress.emit(1, 'Finished calculating data to display')
        if (hasattr(self.parentWindow.ui, 'action_enableLivePreview') and self.parentWindow.ui.action_enableLivePreview.isChecked() == False):
            pass
        else:
            self.parentWindow.dataToDisplay = dataToDisplay
        # self.dataReady.emit(dataToDisplay)
        if not (self.parentWindow.PSDThread.isRunning()):
            # window.PSDWorkerInstance.ADCData = ADCData[window.columnSelect]*1.0
            self.parentWindow.PSDWorkerInstance.ADCData = ADCData.astype(numpy.float64) #ADCData is sent as float64
            self.parentWindow.PSDThread.start()
            self.progress.emit(1, 'Calculating histogram')
            # self.startPSDThread.emit()
        self.stop = time.clock()
        # print "Processing the data took", self.stop-self.start, "s"
        self.dataReady.emit()
        self.finished.emit()

    def createFilter(self, livePreviewFilterBandwidth):
        """Given that SUBSAMPLINGFACTOR reduces the raw data to a lower data rate, the filter cutoff frequency below needs to be adjusted accordingly
        For example, for ADCSAMPLINGRATE = 156.25e6/4, SUBSAMPLINGFACTOR of 40 produces samples roughly every 1us. Therefore, effective signal
        bandwidth reduces to 500kHz. To sample down to 100kHz then requires a cutoff frequency of 0.2 (= 100kHz/500kHz)."""
        self.livePreviewFilterBandwidth = livePreviewFilterBandwidth
        self.b, self.a = scipy.signal.bessel(4, livePreviewFilterBandwidth/self.dictOfConstants['ADCSAMPLINGRATE']*2, 'low')

    def unpackData(self):
        """Returns unpacked ADCData for the selected column"""
        if ('Master' == self.FPGAType):
            self.rawDataUnpacked = numpy.frombuffer(self.rawData, dtype='uint32')
            self.progress.emit(1, 'Unpacking data')
            if (0 == self.parentWindow.columnSelect):
                ADCData = numpy.bitwise_and(self.rawDataUnpacked, 0xfff)
            elif (1 == self.parentWindow.columnSelect):
                ADCData = numpy.bitwise_and(self.rawDataUnpacked, 0xfff000) >> 12
            self.progress.emit(1, 'Processing data for plotting')
            return ADCData
        if ('Slave' == self.FPGAType):
            self.rawData64Bit = numpy.frombuffer(self.rawData, dtype='uint64')
            self.progress.emit(1, 'Unpacking data')
            if (2 == self.parentWindow.columnSelect):
                ADCDataCompressed = numpy.bitwise_and(self.rawData64Bit[0::2], 0xfffffffff)
                ADCData = numpy.zeros((3*numpy.size(ADCDataCompressed),))
                ADCData[0::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff)
                ADCData[1::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff000) >> 12
                ADCData[2::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff000000) >> 24
            elif (3 == self.parentWindow.columnSelect):
                ADCDataCompressed = numpy.bitwise_and(self.rawData64Bit[0::2], 0xfffffff000000000) >> 36
                ADCDataCompressedMSB = numpy.bitwise_and(self.rawData64Bit[1::2], 0xff)
                ADCData = numpy.zeros((3*numpy.size(ADCDataCompressed),))
                ADCData[0::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff)
                ADCData[1::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff000) >> 12
                ADCData[2::3] = (numpy.bitwise_and(ADCDataCompressed, 0xf000000) >> 24) + ADCDataCompressedMSB*16
            elif (4 == self.parentWindow.columnSelect):
                ADCDataCompressed = numpy.bitwise_and(self.rawData64Bit[1::2], 0x00000fffffffff00) >> 8
                ADCData = numpy.zeros((3*numpy.size(ADCDataCompressed),))
                ADCData[0::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff)
                ADCData[1::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff000) >> 12
                ADCData[2::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff000000) >> 24
            self.progress.emit(1, 'Processing data for plotting')
            return ADCData
        elif ('Generic' == self.FPGAType):
            if (self.parentWindow.columnSelect in [0, 1]):
                self.rawDataUnpacked = numpy.frombuffer(self.rawData, dtype='uint32')
                self.progress.emit(1, 'Unpacking data')
                if (0 == self.parentWindow.columnSelect):
                    ADCData = numpy.bitwise_and(self.rawDataUnpacked, 0xfff)
                elif (1 == self.parentWindow.columnSelect):
                    ADCData = numpy.bitwise_and(self.rawDataUnpacked, 0xfff000) >> 12
                self.progress.emit(1, 'Processing data for plotting')
                return ADCData
            elif (self.parentWindow.columnSelect in [2, 3, 4]):
                self.rawData64Bit = numpy.frombuffer(self.rawData, dtype='uint64')
                self.progress.emit(1, 'Unpacking data')
                if (2 == self.parentWindow.columnSelect):
                    ADCDataCompressed = numpy.bitwise_and(self.rawData64Bit[0::2], 0xfffffffff)
                    ADCData = numpy.zeros((3*numpy.size(ADCDataCompressed),))
                    ADCData[0::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff)
                    ADCData[1::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff000) >> 12
                    ADCData[2::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff000000) >> 24
                elif (3 == self.parentWindow.columnSelect):
                    ADCDataCompressed = numpy.bitwise_and(self.rawData64Bit[0::2], 0xfffffff000000000) >> 36
                    ADCDataCompressedMSB = numpy.bitwise_and(self.rawData64Bit[1::2], 0xff)
                    ADCData = numpy.zeros((3*numpy.size(ADCDataCompressed),))
                    ADCData[0::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff)
                    ADCData[1::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff000) >> 12
                    ADCData[2::3] = (numpy.bitwise_and(ADCDataCompressed, 0xf000000) >> 24) + ADCDataCompressedMSB*16
                elif (4 == self.parentWindow.columnSelect):
                    ADCDataCompressed = numpy.bitwise_and(self.rawData64Bit[1::2], 0x00000fffffffff00) >> 8
                    ADCData = numpy.zeros((3*numpy.size(ADCDataCompressed),))
                    ADCData[0::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff)
                    ADCData[1::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff000) >> 12
                    ADCData[2::3] = numpy.bitwise_and(ADCDataCompressed, 0xfff000000) >> 24
                self.progress.emit(1, 'Processing data for plotting')
                return ADCData

class WriteToLogFileWorker(QtCore.QObject):
    finished = QtCore.pyqtSignal()

    def __init__(self, parentWindow, dictOfConstants):
        super(WriteToLogFileWorker, self).__init__()
        self.rawData = Queue.Queue()
        self.f = 0
        self.parentWindow = parentWindow
        self.dictOfConstants = dictOfConstants
        self.defaultDirectory = "./Logfiles" + "/" + datetime.date.today().strftime("%Y%m%d") + "/"

    def writeToLogFile(self):
        start = time.clock()
        if not os.path.exists(self.defaultDirectory):
            os.makedirs(self.defaultDirectory)
        # print len(self.rawData[0])
        self.fileName = self.defaultDirectory + self.parentWindow.dataFileSelected + "_" + datetime.date.today().strftime("%Y%m%d") + "_" + time.strftime("%H%M%S") + ".hex"
        self.f = open(self.fileName, 'a+b')
        # While logging is enabled, write in 1 second bursts. Finish writing everything that is in memory when logging is disabled
        if (self.parentWindow.ui.action_enableLogging.isChecked()):
            for i in range(self.dictOfConstants['REFRESHRATE']):
                self.f.write(self.rawData.get())
        # self.rawData = []
        else:
            for i in range(self.rawData.qsize()):
                self.f.write(self.rawData.get())
        self.f.close()
        self.configFileName = self.fileName[0:len(self.fileName) - 3] + "cfg"
        self.parentWindow.action_saveState_triggered(self.configFileName)
        stop = time.clock()
        print "Writing data to disk took", stop-start, "s"
        if ((stop - start)*1000 > self.dictOfConstants['FRAMEDURATION']):
            print "Probably missed writing some data because it took too long to write the last frame"
        self.finished.emit()
      
class AnalyzeDataWorker(QtCore.QObject):
    finished = QtCore.pyqtSignal()
    
    def __init__(self, parentWindow, dictOfConstants):
        super(AnalyzeDataWorker, self).__init__()
        self.parentWindow = parentWindow
        self.dictOfConstants = dictOfConstants
        self.rawData = 0
        
    def analyzeData(self, detectEdges=True):
        # self.rawData = self.parentWindow.processRawDataWorkerInstance.ADCData2/self.parentWindow.RDCFB/self.dictOfConstants['AAFILTERGAIN'] - self.parentWindow.IDCOffset
        self.baseline = numpy.mean(self.rawData)
        self.parentWindow.baseline = self.baseline
        if (self.parentWindow.threshold is not None):
            self.threshold = self.parentWindow.threshold
        else:
            self.threshold = self.baseline - 5*numpy.std(self.rawData)
            self.parentWindow.threshold = self.threshold
        try:
            self.minimumEventDuration = eval(str(self.parentWindow.ui.lineEdit_minimumEventDuration.text()))*1e-6
            self.minimumEventSamples = self.dictOfConstants['ADCSAMPLINGRATE']*self.minimumEventDuration
        except:
            self.minimumEventDuration = 1e-6
            self.minimumEventSamples = self.dictOfConstants['ADCSAMPLINGRATE']*self.minimumEventDuration
        try:
            self.maximumEventDuration = eval(str(self.parentWindow.ui.lineEdit_maximumEventDuration.text()))*1e-6
            self.maximumEventSamples = self.dictOfConstants['ADCSAMPLINGRATE']*self.maximumEventDuration
        except:
            self.maximumEventDuration = 1e-3
            self.maximumEventSamples = self.dictOfConstants['ADCSAMPLINGRATE']*self.maximumEventDuration
        if (detectEdges == True):
            self.detectEdges()
            
    def detectEdges(self):
        self.eventIndex = (numpy.where(self.rawData < self.threshold)[0]).astype(numpy.int32)
        self.transitions = numpy.diff(self.eventIndex)
        self.edgeBeginIndex = numpy.insert(self.transitions, 0, 2)
        self.edgeEndIndex = numpy.insert(self.transitions, -1, 2)
        self.edgeBegin = self.eventIndex[numpy.where(self.edgeBeginIndex > 1)]
        self.edgeEnd = self.eventIndex[numpy.where(self.edgeEndIndex > 1)]
        
        self.numberOfEvents = len(self.edgeBegin)
        self.limit = self.baseline - 0.9*numpy.std(self.rawData) #Set the upper limit to 5 sigma away from the mean
        for i in range(self.numberOfEvents):
            eb = self.edgeBegin[i]
            while self.rawData[eb] < self.limit and eb > 0:
                eb -= 1
            self.edgeBegin[i] = eb
            
            ee = self.edgeEnd[i]
            while self.rawData[ee] < self.limit:
                ee += 1
                if i == self.numberOfEvents - 1:
                    if ee == len(self.rawData) - 1:
                        self.edgeEnd[i] = 0
                        self.edgeBegin[i] = 0
                        ee = 0
                        break
                elif ee > self.edgeBegin[i + 1]:
                    self.edgeBegin[i+1] = 0
                    self.edgeEnd[i] = 0
                    ee = 0
                    break
            self.edgeEnd[i] = ee
            
        self.edgeBegin = self.edgeBegin[self.edgeBegin != 0]
        self.edgeEnd = self.edgeEnd[self.edgeEnd != 0]
        self.numberOfEvents = len(self.edgeBegin)
        
        # Remove events shorter than duration specified in the GUI
        for i in range(self.numberOfEvents):
            if ((self.edgeEnd[i] - self.edgeBegin[i]) < self.minimumEventSamples) or ((self.edgeEnd[i] - self.edgeBegin[i]) > self.maximumEventSamples):
                self.edgeBegin[i] = 0
                self.edgeEnd[i] = 0
        
        self.edgeBegin = self.edgeBegin[self.edgeBegin != 0]
        self.edgeEnd = self.edgeEnd[self.edgeEnd != 0]
        self.numberOfEvents = len(self.edgeBegin)
        
        self.eventIndex = numpy.empty_like(self.edgeBegin, dtype='object') #Overwrites previous self.eventIndex
        self.eventValue = numpy.empty_like(self.edgeBegin, dtype='object') 
        self.deltaI = numpy.empty_like(self.edgeBegin, dtype='object') 
        self.meanDeltaI = numpy.empty_like(self.edgeBegin, dtype=numpy.float64)
        self.dwellTime = numpy.empty_like(self.edgeBegin, dtype=numpy.float64)
        
        for i in range(self.numberOfEvents):
            self.eventIndex[i] = numpy.arange(self.edgeBegin[i], self.edgeEnd[i]+1)
            self.eventValue[i] = self.rawData[self.eventIndex[i]]
            self.deltaI[i] = self.baseline - self.rawData[self.eventIndex[i]]
            self.meanDeltaI[i] = numpy.mean(self.deltaI[i], dtype=numpy.float64)
            self.dwellTime[i] = (self.edgeEnd[i] - self.edgeBegin[i]).astype(numpy.float64)/self.dictOfConstants['ADCSAMPLINGRATE']
            
        # print self.edgeBegin, self.edgeEnd, self.eventIndex
        self.parentWindow.edgeBegin = self.edgeBegin
        self.parentWindow.edgeEnd = self.edgeEnd
        self.parentWindow.numberOfEvents = self.numberOfEvents
        self.parentWindow.eventIndex = self.eventIndex
        self.parentWindow.eventValue = self.eventValue
        self.parentWindow.deltaI = self.deltaI
        self.parentWindow.meanDeltaI = self.meanDeltaI
        self.parentWindow.dwellTime = self.dwellTime
        
        self.finished.emit()
        #self.analyze_cusum_modified()
        
    def analyze_cusum(self):
        '''Analyzes the waveform using the CUSUM algorithm. Adapted from PyPore implementation but only handles downward spikes'''
        
        #Initializations
        i = 0
        eventNumber = 0
        dataPoint = self.rawData[i]
        nData = len(self.rawData)
        baseline = dataPoint
        variance = numpy.var(self.rawData[0:100])
        variance_baseline = baseline
        isEvent = False
        
        maxNumberOfEvents = 1000
        
        self.edgeBegin = numpy.empty(maxNumberOfEvents)
        self.edgeEnd = numpy.empty(maxNumberOfEvents)
        self.eventValue = numpy.empty([maxNumberOfEvents, self.maximumEventSamples], dtype='object')
        self.meanEventValue = numpy.empty([maxNumberOfEvents, self.maximumEventSamples], dtype='object')
        self.dwellTime = numpy.empty([maxNumberOfEvents, self.maximumEventSamples], dtype='object')
        
        threshold_start = 5 * numpy.sqrt(variance)
        threshold_end = 1 * numpy.sqrt(variance)
        
        while i < nData:
            dataPoint = self.rawData[i]
            
            if (dataPoint < baseline - threshold_start):
                isEvent = True
                self.edgeBegin[eventNumber] = i
            
            if isEvent:
                isEvent = False
                done = False
                meanEstimate = dataPoint
                nLevels = 0
                sn = sp = Sn = Sp = Gn = Gp = 0
                varianceEstimate = variance
                threshold_end = 1 * numpy.sqrt(variance)
                
                delta = numpy.abs(meanEstimate - baseline)/2.
                minIndexP = minIndexN = i
                
                eventArea = dataPoint
                event_i = i
                ko = i
                
                levelSum = dataPoint
                levelSumMinP = dataPoint
                levelSumMinN = dataPoint
                previousLevelStart = event_i
                
                while not done and event_i - self.edgeBegin[eventNumber] < self.maximumEventSamples:
                    event_i += 1
                    dataPoint = self.rawData[event_i]
                    
                    if dataPoint >= baseline - threshold_end:
                        done = True
                        self.edgeEnd[eventNumber] = event_i
                        break
                        
                    newMean = meanEstimate + (dataPoint - meanEstimate)/(1 + event_i - ko)
                    varianceEstimate = ((event_i - ko) * varianceEstimate + (dataPoint - meanEstimate) * (dataPoint - newMean))/(1 + event_i - ko)
                    meanEstimate = newMean
                    if varianceEstimate > 0:
                        sp = (delta/varianceEstimate) * (dataPoint - meanEstimate - delta/2.)
                        sn = -(delta/varianceEstimate) * (dataPoint - meanEstimate + delta/2.)
                    elif delta == 0:
                        sp = 0
                        sn = 0
                    Sp += sp
                    Sn += sn
                    Gp = numpy.maximum(0., Gp + sp)
                    Gn = numpy.maximum(0., Gn + sn)
                    levelSum += dataPoint
                    
                    if Sp <= 0:
                        Sp = 0
                        minIndexP = event_i
                        levelSumMinP = levelSum
                    if Sn <= 0:
                        Sn = 0
                        minIndexN = event_i
                        levelSumMinN = levelSum
                    h = delta/numpy.sqrt(varianceEstimate)

                    if Gp > h or Gn > h:
                        if Gp > h:
                            minIndex = minIndexP
                            levelSum = levelSumMinP
                        else:
                            minIndex = minIndexN
                            levelSum = levelSumMinN
                        self.dwellTime[eventNumber][nLevels] = (minIndex + 1 - ko)*1.0/self.dictOfConstants['ADCSAMPLINGRATE']
                        self.meanEventValue[eventNumber][nLevels] = levelSum/self.dwellTime[eventNumber][nLevels]*self.dictOfConstants['ADCSAMPLINGRATE']
                        nLevels += 1
                        sn = sp = Sn = Sp = Gn = Gp = 0
                        ko = event_i = minIndex + 1
                        minIndexP = minIndexN = event_i
                        previousLevelStart = event_i
                        meanEstimate = self.rawData[event_i]
                        levelSum = levelSumMinP = levelSumMinN = meanEstimate
                        
                i = self.edgeEnd[eventNumber].astype(int)
                if self.edgeEnd[eventNumber] > previousLevelStart:
                    self.dwellTime[eventNumber][nLevels] = (self.edgeEnd[eventNumber] - previousLevelStart)*1.0/self.dictOfConstants['ADCSAMPLINGRATE']
                    self.meanEventValue[eventNumber][nLevels] = levelSum/self.dwellTime[eventNumber][nLevels]*self.dictOfConstants['ADCSAMPLINGRATE']
                    nLevels += 1
                    
                if done and self.edgeEnd[eventNumber] - self.edgeBegin[eventNumber] > self.minimumEventSamples:
                    if self.edgeEnd[eventNumber] - self.edgeBegin[eventNumber] < 2*self.minimumEventSamples:
                        nLevels = 0
                        self.meanEventValue[eventNumber][nLevels] = numpy.min(self.rawData[self.edgeBegin[eventNumber]:self.edgeEnd[eventNumber]+1])
                        self.dwellTime[eventNumber][nLevels] = (self.edgeEnd[eventNumber]-self.edgeBegin[eventNumber])*1.0/self.dictOfConstants['ADCSAMPLINGRATE']
                        
                eventNumber += 1
                        
            baseline = baseline * 0.93 + (1 - 0.93) * dataPoint #Adaptive basline
            variance_baseline = 0.99 * variance_baseline + (1 - 0.99) * dataPoint
            variance = 0.99 * variance + (1 - 0.99) * numpy.power(dataPoint - variance_baseline, 2) #Adaptive baseline
            i += 1
            
            if i % 100000 == 0:
                print i, eventNumber
                
        self.eventIndex = numpy.empty_like(self.edgeBegin, dtype='object')
        for i in range(eventNumber):
            self.eventIndex[i] = numpy.arange(self.edgeBegin[i], self.edgeEnd[i]+1)
        
        self.parentWindow.edgeBegin = self.edgeBegin[:eventNumber].astype(int)
        self.parentWindow.edgeEnd = self.edgeEnd[:eventNumber].astype(int)
        self.numberOfEvents = int(eventNumber)
        self.parentWindow.numberOfEvents = self.numberOfEvents
        self.parentWindow.eventIndex = self.eventIndex
        
        self.finished.emit()
        
    def analyze_cusum_modified(self):
        #Initializations
        i = 0
        j = 0
        eventNumber = 0
        dataPoint = self.rawData[i]
        nData = len(self.rawData)
        baseline = dataPoint
        variance = numpy.var(self.rawData[0:100])
        variance_baseline = baseline
        isEvent = False
        localEventIndex = numpy.hstack(self.eventIndex)
        
        maxNumberOfEvents = 1000
        
        # self.edgeBegin = numpy.empty(maxNumberOfEvents)
        # self.edgeEnd = numpy.empty(maxNumberOfEvents)
        self.eventValue = numpy.empty([maxNumberOfEvents, self.maximumEventSamples], dtype='object')
        self.meanEventValue = numpy.empty([maxNumberOfEvents, self.maximumEventSamples], dtype='object')
        self.dwellTime = numpy.empty([maxNumberOfEvents, self.maximumEventSamples], dtype='object')
        
        threshold_start = 5 * numpy.sqrt(variance)
        threshold_end = 1 * numpy.sqrt(variance)
        
        while j < (len(localEventIndex)):
            i = localEventIndex[j]
            dataPoint = self.rawData[i]
                        
            if True:
                done = False
                meanEstimate = dataPoint
                nLevels = 0
                sn = sp = Sn = Sp = Gn = Gp = 0
                varianceEstimate = variance
                threshold_end = 1 * numpy.sqrt(variance)
                
                delta = numpy.abs(meanEstimate - baseline)/2.
                minIndexP = minIndexN = i
                
                eventArea = dataPoint
                event_i = i
                ko = i
                
                levelSum = dataPoint
                levelSumMinP = dataPoint
                levelSumMinN = dataPoint
                previousLevelStart = event_i
                
                while not done:
                    event_i += 1
                    dataPoint = self.rawData[event_i]
                    
                    if dataPoint >= baseline - threshold_end:
                        done = True
                        break
                        
                    newMean = meanEstimate + (dataPoint - meanEstimate)/(1 + event_i - ko)
                    varianceEstimate = ((event_i - ko) * varianceEstimate + (dataPoint - meanEstimate) * (dataPoint - newMean))/(1 + event_i - ko)
                    meanEstimate = newMean
                    if varianceEstimate > 0:
                        sp = (delta/varianceEstimate) * (dataPoint - meanEstimate - delta/2.)
                        sn = -(delta/varianceEstimate) * (dataPoint - meanEstimate + delta/2.)
                    elif delta == 0:
                        sp = 0
                        sn = 0
                    Sp += sp
                    Sn += sn
                    Gp = numpy.maximum(0., Gp + sp)
                    Gn = numpy.maximum(0., Gn + sn)
                    levelSum += dataPoint
                    
                    if Sp <= 0:
                        Sp = 0
                        minIndexP = event_i
                        levelSumMinP = levelSum
                    if Sn <= 0:
                        Sn = 0
                        minIndexN = event_i
                        levelSumMinN = levelSum
                    h = delta/numpy.sqrt(varianceEstimate)

                    if Gp > h or Gn > h:
                        if Gp > h:
                            minIndex = minIndexP
                            levelSum = levelSumMinP
                        else:
                            minIndex = minIndexN
                            levelSum = levelSumMinN
                        self.dwellTime[eventNumber][nLevels] = (minIndex + 1 - ko)*1.0/self.dictOfConstants['ADCSAMPLINGRATE']
                        self.meanEventValue[eventNumber][nLevels] = levelSum/self.dwellTime[eventNumber][nLevels]*self.dictOfConstants['ADCSAMPLINGRATE']
                        nLevels += 1
                        sn = sp = Sn = Sp = Gn = Gp = 0
                        ko = event_i = minIndex + 1
                        minIndexP = minIndexN = event_i
                        previousLevelStart = event_i
                        meanEstimate = self.rawData[event_i]
                        levelSum = levelSumMinP = levelSumMinN = meanEstimate
                        
                # i = self.edgeEnd[eventNumber].astype(int)
                if self.edgeEnd[eventNumber] > previousLevelStart:
                    self.dwellTime[eventNumber][nLevels] = (self.edgeEnd[eventNumber] - previousLevelStart)*1.0/self.dictOfConstants['ADCSAMPLINGRATE']
                    self.meanEventValue[eventNumber][nLevels] = levelSum/self.dwellTime[eventNumber][nLevels]*self.dictOfConstants['ADCSAMPLINGRATE']
                    nLevels += 1
                    
                if done and self.edgeEnd[eventNumber] - self.edgeBegin[eventNumber] > self.minimumEventSamples:
                    if self.edgeEnd[eventNumber] - self.edgeBegin[eventNumber] < 2*self.minimumEventSamples:
                        nLevels = 0
                        self.meanEventValue[eventNumber][nLevels] = numpy.min(self.rawData[self.edgeBegin[eventNumber]:self.edgeEnd[eventNumber]+1])
                        self.dwellTime[eventNumber][nLevels] = (self.edgeEnd[eventNumber]-self.edgeBegin[eventNumber])*1.0/self.dictOfConstants['ADCSAMPLINGRATE']
                        
                eventNumber += 1
                        
            # baseline = baseline * 0.93 + (1 - 0.93) * dataPoint #Adaptive basline
            # variance_baseline = 0.99 * variance_baseline + (1 - 0.99) * dataPoint
            # variance = 0.99 * variance + (1 - 0.99) * numpy.power(dataPoint - variance_baseline, 2) #Adaptive baseline
            j = numpy.where(localEventIndex == event_i)[0] + 1
            # print eventNumber, i, j, len(localEventIndex), baseline, threshold_end
                
        # self.eventIndex = numpy.empty_like(self.edgeBegin, dtype='object')
        # for i in range(eventNumber):
            # self.eventIndex[i] = numpy.arange(self.edgeBegin[i], self.edgeEnd[i]+1)
        
        print "Finishing"
        
        self.finished.emit()
                    


#!/usr/bin/env python
# -*- coding: utf8 -*-

"""

This program is for finding events in files and displaying the results.
"""
import sys
import os

from PySide import QtCore, QtGui  # Must import PySide stuff before pyqtgraph so pyqtgraph knows
# to use PySide instead of PyQt

# The rest of the imports can be found below in _longImports


def _long_imports(**kwargs):
    """
    Loads imports and updates the splash screen with information.
    """
    # append the src directory to the PYTHONPATH, i.e. '../../' = 'src/'
    src_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
    if not src_dir in sys.path:
        sys.path.append(src_dir)

    global AnalyzeDataThread, PlotThread, pg, pgc, LayoutWidget, linspace, np, \
        EventAnalysisWidget, ed, EventFindingTab, EventViewingTab, EventAnalysisTab, \
        FileViewerTab

    update_splash = False
    if 'splash' in kwargs and 'app' in kwargs:
        update_splash = True
        splash = kwargs['splash']
        app = kwargs['app']

    if update_splash:
        splash.showMessage("Importing PyQtGraph...", alignment=QtCore.Qt.AlignBottom)
        app.processEvents()
    import pyqtgraph as pg
    import pyqtgraph.console as pgc
    from pyqtgraph.widgets.LayoutWidget import LayoutWidget

    if update_splash:
        splash.showMessage("Importing SciPy...", alignment=QtCore.Qt.AlignBottom)
        app.processEvents()
    from scipy import linspace

    if update_splash:
        splash.showMessage("Importing NumPy...", alignment=QtCore.Qt.AlignBottom)
        app.processEvents()
    import numpy as np

    if update_splash:
        splash.showMessage("Compiling Cython imports... DataFileOpener", alignment=QtCore.Qt.AlignBottom)
        app.processEvents()
    from widgets.event_viewing_tab import EventViewingTab
    from widgets.event_analysis_tab import EventAnalysisTab
    from widgets.event_finding_tab import EventFindingTab
    from widgets.file_viewer_tab import FileViewerTab

    if update_splash:
        splash.showMessage("Compiling Cython imports... EventFinder", alignment=QtCore.Qt.AlignBottom)
        app.processEvents()
    from my_threads import AnalyzeDataThread, PlotThread

    if update_splash:
        splash.showMessage("Importing Event Database", alignment=QtCore.Qt.AlignBottom)
        app.processEvents()
    import pypore.filetypes.event_database as ed


class MyMainWindow(QtGui.QMainWindow):
    def __init__(self, app, parent=None):
        super(MyMainWindow, self).__init__()

        self.events = []  # holds the events from the most recent analysis run
        self.app = app

        pg.setConfigOption('leftButtonPan', False)

        self.open_dir = '../../data'

        self.thread_pool = []

        self.setWindowTitle('Translocation Event Analysis')

        self._create_menu()
        self._create_main_frame()
        self._create_status_bar()
        self._on_current_tab_changed(0)

    def open_data_files(self):
        """
        Opens file dialog box, adds names of files to open to list
        """
        file_names = QtGui.QFileDialog.getOpenFileNames(self,
                                                        'Open data file',
                                                        self.open_dir,
                                                        "All files *(*);;"
                                                        "Pypore data files *.h5(*.h5);;"
                                                        "Heka files *.hkd(*.hkd);;"
                                                        "Chimera files *.log(*.log);;"
                                                        "Gabys files *.mat(*.mat);;"
                                                        "CNP2 *.hex(*.hex)")[0]
        if len(file_names) > 0:
            self.main_tabwig.currentWidget().open_data_files(file_names)

    def open_event_databases(self):
        """
        Opens file dialog box, add names of event database files to open list
        """
        file_names = QtGui.QFileDialog.getOpenFileNames(self, 'Open event database', self.open_dir, '*.h5')[0]

        if len(file_names) > 0:
            self.main_tabwig.currentWidget().open_event_databases(file_names)

    def set_status(self, text):
        """
        Sets the status text.

        :param StringType text: Text to display in the status bar.
        """
        self.status_text.setText(text)

    def _process_events(self):
        self.app.processEvents()

    def _create_main_frame(self):
        """
        Helper to initialize_c the main gui frame.
        """
        file_viewer_tab = FileViewerTab(self)

        event_finding_tab = EventFindingTab(self)
        event_finding_tab.set_on_status_update_callback(self.set_status)
        event_finding_tab.set_process_events_callback(self._process_events)

        event_viewer_tab = EventViewingTab(self)

        event_analysis_tab = EventAnalysisTab(self)

        # Layout holding everything
        self.main_tabwig = QtGui.QTabWidget()
        self.main_tabwig.addTab(file_viewer_tab, 'File Viewer')
        self.main_tabwig.addTab(event_finding_tab, 'Event Finding')
        self.main_tabwig.addTab(event_viewer_tab, 'Event View')
        self.main_tabwig.addTab(event_analysis_tab, 'Event Analysis')
        self.main_tabwig.setMinimumSize(1000, 550)

        self.main_tabwig.currentChanged.connect(self._on_current_tab_changed)

        text = """*********************
Welcome to pyporegui!

If you are unfamiliar with the python console, feel free to ignore this console.

However, you can use this console to interact with your data and the gui!
Type globals() to see globally defined variabels.
Type locals() to see application-specific variables.

The current namespace should include:
    np        -    numpy
    pg        -    pyqtgraph
    ed        -    pypore.eventDatabase
    currentPlot -  Top plot in the event finding tab.
*********************"""

        namespace = {'np': np, 'pg': pg, 'ed': ed, 'currentPlot': event_finding_tab.plot_widget}
        self.console = pgc.ConsoleWidget(namespace=namespace, text=text)

        frame = QtGui.QSplitter()
        frame.setOrientation(QtCore.Qt.Vertical)
        frame.addWidget(self.main_tabwig)
        frame.addWidget(self.console)

        self.setCentralWidget(frame)

    def _on_current_tab_changed(self, index):
        """
        Callback for when the user changes the current tab to index position.
        """
        wig = self.main_tabwig.widget(index)
        # first disable all of the tab-dependent actions
        for action in self.open_menu.actions():
            action.setEnabled(False)
        # Then add actions that this widget requests.
        if hasattr(wig, 'open_data_files'):
            self.load_data_file_action.setEnabled(True)
        if hasattr(wig, 'open_event_databases'):
            self.load_events_database_action.setEnabled(True)

    def _create_status_bar(self):
        """
        Creates baseline_filter_parameter status bar with baseline_filter_parameter text widget.
        """
        self.status_text = QtGui.QLabel("")
        self.statusBar().addWidget(self.status_text, 1)

    def _create_menu(self):
        """
        Creates File menu with Open
        """
        self.file_menu = self.menuBar().addMenu("&File")

        self.open_menu = self.file_menu.addMenu("&Open")

        self.load_data_file_action = self.create_action("&Open Data File",
                                                        shortcut="Ctrl+O", slot=self.open_data_files,
                                                        tip="Open data Files")
        self.load_data_file_action.setEnabled(False)
        self.load_events_database_action = self.create_action("&Open Events Database",
                                                              shortcut="Ctrl+E", slot=self.open_event_databases,
                                                              tip="Open Events Database")
        self.load_events_database_action.setEnabled(False)
        quit_action = self.create_action("&Quit", slot=self.close,
                                         shortcut="Ctrl+Q", tip="Close the application")

        self.add_actions(self.open_menu, (self.load_data_file_action, self.load_events_database_action))

        self.add_actions(self.file_menu,
                         (None, quit_action))

    #         self.help_menu = self.menuBar().addMenu("&Help")
    #         about_action = self.create_action("&About",
    #             shortcut='F1', slot=self.on_about,
    #             tip='About the demo')
    #
    #         self.add_actions(self.help_menu, (about_action,))

    @staticmethod
    def add_actions(target, actions):
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)

    def create_action(self, text, slot=None, shortcut=None,
                      icon=None, tip=None, checkable=False,
                      signal="triggered()"):
        """
        Create a :py:class:`QtGui.QAction`.
        :param text: Text to show in the file menu
        """
        action = QtGui.QAction(text, self)
        if icon is not None:
            action.setIcon(QtGui.QIcon(":/%s.png" % icon))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            self.connect(action, QtCore.SIGNAL(signal), slot)
        if checkable:
            action.setCheckable(True)
        return action

    def clean_threads(self):
        for w in self.thread_pool:
            w.cancel()
            # w.wait()
            self.thread_pool.remove(w)


def start():
    app = QtGui.QApplication(sys.argv)
    pix_map = QtGui.QPixmap('splash.png')
    splash = QtGui.QSplashScreen(pix_map, QtCore.Qt.WindowStaysOnTopHint)
    splash.show()
    _long_imports(splash=splash, app=app)
    splash.showMessage("Creating main window...", alignment=QtCore.Qt.AlignBottom)
    app.processEvents()
    ex = MyMainWindow(app)
    ex.show()
    splash.finish(ex)
    app.exec_()
    ex.clean_threads()


if __name__ == '__main__':
    start()
else:
    # If we are running from a tests, name != main, and we'll need to import the long imports now.
    _long_imports()



from PySide import QtGui
from pypore.data_file_opener import prepare_data_file
from pypore.i_o import get_reader_from_filename
from pyporegui._thread_manager import _ThreadManager
from pyporegui.file_items import DataFileListItem, FileListItem


class BaseQSplitter(_ThreadManager, QtGui.QSplitter):
    def __init__(self, parent):
        super(BaseQSplitter, self).__init__(parent)

        self.on_status_update_callback = None
        self.process_events_callback = None

        left_widget = self._create_left_widget(parent)
        right_widget = self._create_right_widget(parent)

        self.addWidget(left_widget)
        self.addWidget(right_widget)

    def set_process_events_callback(self, callback):
        """
        Sets a callback for when the EventFindingTab requests app.processEvents

        :param MethodType callback:
        """
        self.process_events_callback = callback

    def _dispatch_process_events(self):
        """
        Calls the process_events_callback if it is not None.
        """
        if self.process_events_callback is not None:
            self.process_events_callback()

    def set_on_status_update_callback(self, callback):
        """
        Sets a callback for when the EventFinderTab's status is updated. The status is a String of information
        about the current state of the EventFinderTab.

        :param MethodType callback: Callback method for when the EventFinderTab's status is updated.
                                    This callback must accept one string parameter.
        """
        self.on_status_update_callback = callback

    def _dispatch_status_update(self, text):
        """
        Dispatches text to the on_status_update_callback if the on_status_update_callback is not None.
        """
        if self.on_status_update_callback is not None:
            self.on_status_update_callback(text)


class BaseQSplitterDataFile(BaseQSplitter):
    def __init__(self, parent):
        super(BaseQSplitterDataFile, self).__init__(parent)

    def open_data_files(self, file_names=None):
        """
        Analyzes the files for correctness, then adds them to the list widget.

        :param ListType<StringType> file_names: The file names to be included in the list widget. If not included,
                                                this function will use a QtGui.QFileDialog.getOpenFileNames to open
                                                files.
        :returns: BooleanType -- **True** if files were opened, **False** otherwise.
        """
        if file_names is None:
            file_names = QtGui.QFileDialog.getOpenFileNames(self,
                                                            'Open data file',
                                                            '.',
                                                            "All types(*.h5 *.hkd *.log *.mat);;"
                                                            "Pypore data files *.h5(*.h5);;"
                                                            "Heka files *.hkd(*.hkd);;"
                                                            "Chimera files *.log(*.log);;Gabys files *.mat(*.mat)")[0]
        if len(file_names) > 0:
            self.file_list_widget.clear()
        else:
            return
        are_files_opened = False
        open_dir = None
        for w in file_names:
            reader = get_reader_from_filename(w)
            # if 'error' in params:
            # TODO implement error handling in readers
            # pass
            # else:
            reader.close()
            are_files_opened = True
            item = FileListItem(w)
            open_dir = item.get_directory()
            self.file_list_widget.addItem(item)

        return are_files_opened

    def _create_list_form(self):
        self.file_list_widget = QtGui.QListWidget()
        self.file_list_widget.setMaximumHeight(100)

        self.file_list_widget.itemSelectionChanged.connect(self._on_file_item_selection_changed)
        self.file_list_widget.itemDoubleClicked.connect(self._on_file_item_doubleclick)
        self.file_list_widget.setMaximumHeight(100)

        file_options = QtGui.QFormLayout()
        file_options.addRow('Data Files:', self.file_list_widget)

        return file_options

    def _create_left_widget(self, parent=None):
        return self._create_list_form()



"""

"""

import unittest
import os

import numpy as np

from pypore.i_o import get_reader_from_filename
from pypore.file_converter import convert_file
from pypore.i_o.data_file_reader import DataFileReader
from pypore.tests.util import _test_file_manager
import pypore.sampledata.testing_files as tf


DIRECTORY = os.path.dirname(os.path.abspath(__file__))


class TestFileConverter(unittest.TestCase):
    @_test_file_manager(DIRECTORY)
    def test_convert_file_set_output_filename(self, filename):
        """
        Tests that the output filename can be set correctly.
        """
        data_filename = tf.get_abs_path('chimera_1event.log')

        output_filename = convert_file(data_filename, output_filename=filename)

        # Test that we can set the output filename
        self.assertEqual(output_filename, filename, "output_filename not correct")

    @_test_file_manager(DIRECTORY)
    def test_convert_chimera_file_equality(self, filename):
        """
        Test that the original/converted matrices and sample rates are the same for one-channel data.
        """
        data_filename = tf.get_abs_path('chimera_1event.log')

        output_filename = convert_file(data_filename, output_filename=filename)

        orig_reader = get_reader_from_filename(data_filename)
        orig_data_all = orig_reader.get_all_data()
        orig_sample_rate = orig_reader.get_sample_rate()

        self.assertEqual(len(orig_data_all), 1)

        out_reader = DataFileReader(output_filename)
        out_data_all = out_reader.get_all_data()
        out_sample_rate = out_reader.get_sample_rate()

        self.assertEqual(len(out_data_all), 1)

        orig_data = orig_data_all[0]
        out_data = out_data_all[0]

        # assert sample rates are equal
        self.assertAlmostEqual(1.0 * orig_sample_rate / out_sample_rate, 1, 4)

        # assert the two arrays are equal
        np.testing.assert_array_equal(orig_data, out_data)

        orig_reader.close()
        out_reader.close()


from pypore.file_converter import concat_files
from pypore.file_converter import SamplingRatesMismatchError


class TestConcatFiles(unittest.TestCase):
    def test_default_output_filename(self):
        """
        Tests that the default output filename is correctly generated from the input file names.
        """
        file_names = [tf.get_abs_path('chimera_1event_2levels.log'), tf.get_abs_path('chimera_1event.log')]

        output_filename = concat_files(file_names)

        self.assertTrue(os.path.exists(output_filename))

        # Check that it's saved in the current directory
        self.assertEqual(output_filename[0:len('chimera_1event')], 'chimera_1event')

        # Check the correct file extension
        self.assertEqual(output_filename[-len('.h5'):], '.h5')

        self.assertIn('_concatenated_', output_filename,
                      "Default output filename ''{0}'' should contain ''_concatenated_''")

        os.remove(output_filename)

    @_test_file_manager(DIRECTORY)
    def test_set_output_filename(self, filename):
        file_names = [tf.get_abs_path('chimera_1event_2levels.log'), tf.get_abs_path('chimera_1event.log')]

        output_filename = concat_files(file_names, output_filename=filename)

        self.assertEqual(output_filename, filename)
        self.assertTrue(os.path.exists(output_filename))

    @_test_file_manager(DIRECTORY)
    def test_different_sample_rate_no_resample(self, filename):
        """
        Tests that an error is thrown if the files have different sample rates and we do not want to resample.
        """

        file_names = [tf.get_abs_path('heka_1.5s_mean5.32p_std2.76p.hkd'), tf.get_abs_path('chimera_1event.log')]

        self.assertRaises(SamplingRatesMismatchError, concat_files, file_names, output_filename=filename)

    def test_single_file_fail(self):
        """
        Tests that an exception is thrown if less than 2 file names are passed in the list.
        """
        file_names = ['hi.txt']

        self.assertRaises(ValueError, concat_files, file_names)
        self.assertRaises(ValueError, concat_files, [])

    @_test_file_manager(DIRECTORY)
    def test_original_files_unmodified(self, filename):
        file1 = tf.get_abs_path('chimera_1event_2levels.log')
        file2 = tf.get_abs_path('chimera_1event.log')

        reader1 = get_reader_from_filename(file1)
        reader2 = get_reader_from_filename(file2)

        sample_rate1_orig = reader1.get_sample_rate()
        sample_rate2_orig = reader2.get_sample_rate()

        data1_orig = reader1.get_all_data()[0]
        data2_orig = reader2.get_all_data()[0]

        reader1.close()
        reader2.close()

        concat_files([file1, file2], output_filename=filename)

        reader1 = get_reader_from_filename(file1)
        reader2 = get_reader_from_filename(file2)

        reader_out = get_reader_from_filename(filename)

        sample_rate1_final = reader1.get_sample_rate()
        sample_rate2_final = reader2.get_sample_rate()

        self.assertEqual(sample_rate1_final, sample_rate1_orig,
                         "Sample rate changed. Was {0}, now {1}.".format(sample_rate1_orig, sample_rate1_final))
        self.assertEqual(sample_rate2_final, sample_rate2_orig,
                         "Sample rate changed. Was {0}, now {1}.".format(sample_rate2_orig, sample_rate2_final))

        data1 = reader1.get_all_data()[0]
        data2 = reader2.get_all_data()[0]

        np.testing.assert_array_equal(data1, data1_orig)
        np.testing.assert_array_equal(data2, data2_orig)

        reader1.close()
        reader2.close()
        reader_out.close()

    @_test_file_manager(DIRECTORY)
    def test_correct_data(self, filename):
        file1 = tf.get_abs_path('chimera_1event_2levels.log')
        file2 = tf.get_abs_path('chimera_1event.log')

        concat_files([file1, file2], output_filename=filename)

        reader1 = get_reader_from_filename(file1)
        reader2 = get_reader_from_filename(file2)

        reader_out = get_reader_from_filename(filename)

        sample_rate1 = reader1.get_sample_rate()
        sample_rate2 = reader2.get_sample_rate()
        sample_rate_out = reader_out.get_sample_rate()

        self.assertEqual(sample_rate_out, sample_rate1,
                         "Unexpected sample rate. Should be {0}, was {1}.".format(sample_rate1, sample_rate_out))
        self.assertEqual(sample_rate_out, sample_rate2,
                         "Unexpected sample rate. Should be {0}, was {1}.".format(sample_rate2, sample_rate_out))

        data1 = reader1.get_all_data()[0]
        data2 = reader2.get_all_data()[0]

        data_out_should_be = np.zeros(data1.size + data2.size)
        data_out_should_be[:data1.size] = data1[:]
        data_out_should_be[data1.size:] = data2[:]

        data_out = reader_out.get_all_data()[0]

        np.testing.assert_array_equal(data_out, data_out_should_be)

        reader1.close()
        reader2.close()
        reader_out.close()


if __name__ == "__main__":
    unittest.main()




#!/usr/bin/env python
import os
import sys

import numpy
import subprocess

CLASSIFIERS = """\
Development Status :: 2 - Pre-Alpha
Intended Audience :: Science/Research
License :: OSI Approved :: Apache Software License
Programming Language :: Python
Programming Language :: Python :: 2.7
Programming Language :: Cython
Topic :: Scientific/Engineering
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: Unix
Operating System :: MacOS
"""

with open('README.rst') as f:
    long_description = f.read()


def is_package(path):
    return os.path.isfile(os.path.join(path, '__init__.py'))


def find_extensions(root):
    """
    Find all the .c files under root directory.
    yields Extension's

    Only yields C files that are in a python package (aka in dir with __init__.py).

    :param string root: root directory to search.
    :returns: Yields a distutils.extension.Extension every time it finds a C file.
    """
    from distutils.extension import Extension

    for path, directories, files in os.walk(root):
        if is_package(path):
            for filename in files:
                if filename.endswith('.c'):
                    full_name = os.path.join(path, filename)
                    module_name = filename[:-len('.c')]
                    full_module_name = get_package_name_from_path(path) + '.' + module_name
                    yield Extension(full_module_name, sources=[full_name])


def get_package_name_from_path(package_path):
    """
    :param str package_path: path of a package
    :returns: string -- full package name, including super packages\
                for a give package_path that is already a package.
    """
    if not is_package(package_path):
        raise ValueError('parameter not a package')
    up_path, end = os.path.split(package_path)
    ret = end
    while end != '' and is_package(up_path):
        up_path, end = os.path.split(up_path)
        ret = end + '.' + ret
    return ret


def find_packages(root):
    """
    Find all the packages under root directory.

    :param string root: root directory to search
    """
    for path, directories, files in os.walk(root):
        if is_package(path):
            yield get_package_name_from_path(path)


def git_version():
    """
    :returns: the git revision as a string.
    """

    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on Win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(cmd, stdout=subprocess.PIPE, env=env).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'rev-parse', 'HEAD'])
        git_revision = out.strip().decode('ascii')
    except OSError:
        git_revision = "Unknown"

    return git_revision


MAJOR = 0
MINOR = 0
MACRO = 3
IS_RELEASE = False
VERSION = '%d.%d.%d' % (MAJOR, MINOR, MACRO)


def get_version_info():
    full_version = VERSION
    if os.path.exists('.git'):
        git_revision = git_version()
    elif os.path.exists('src/pypore/version.py'):
        # must be a source distribution, use existing version file.
        # load it as a separate module to not load pypore/__init__.py
        import imp

        version = imp.load_source('pypore.version', 'src/pypore/version.py')
        git_revision = version.git_revision
    else:
        git_revision = "Unknown"

    if not IS_RELEASE:
        # If we are deploying from the Travis CI server, then add a time string to the version,
        # Hopefully it increments better.
        # TODO change this to 'git describe' once we tag a version.
        if 'TRAVIS_PYTHON_VERSION' in os.environ:
            import time
            time_str = time.strftime('%Y%m%d%H%M%S')
        else:
            time_str = ''

        full_version += '.dev' + time_str

    return full_version, git_revision


def write_version_py(filename_pypore='src/pypore/version.py', filename_pyporegui='src/pyporegui/version.py'):
    """
    Rewrites the src/pypore/version.py and src/pyporegui/version.py files.
    """
    text = """
# THIS FILE IS GENERATED FROM SETUP.PY
short_version = '%(version)s'
version = '%(version)s'
full_version = '%(full_version)s'
git_revision = '%(git_revision)s'
release = %(is_release)s

if not release:
    version = full_version
"""
    full_version, git_revision = get_version_info()

    for filename in (filename_pypore, filename_pyporegui):

        a = open(filename, 'w')
        try:
            a.write(text % {'version': VERSION,
                            'full_version': full_version,
                            'git_revision': git_revision,
                            'is_release': str(IS_RELEASE)})
        finally:
            a.close()


def generate_cython():
    """
    Calls tools/cythonize.py to Cythonize .pyx files to C files.
    """
    cwd = os.path.abspath(os.path.dirname(__file__))
    print("Cythonizing sources")
    p = subprocess.call([sys.executable,
                         os.path.join(cwd, 'tools', 'cythonize.py'),
                         'src/pypore'],
                        cwd=cwd)
    if p != 0:
        raise RuntimeError("Running cythonize failed!")


def _get_version_from_py():
    """
    Reads the 'version' from the python file.
    """
    import imp
    module = imp.load_source('pypore.version', 'src/pypore/version.py')
    return module.version


def setup_package():
    # rewrite the version file every time
    write_version_py()

    metadata = dict(
        name='pypore',
        description='Pythonic/Cythonic Nanopore Translocation Analysis',
        long_description=long_description,
        scripts=['bin/filterfiles.py', 'bin/pypore_batch_csv.py'],
        author='Will Parkin',
        author_email='wmparkin@gmail.com',
        url='http://parkin.github.io/pypore/',
        requires=['numpy', 'scipy', 'tables', 'PySide', 'pyqtgraph'],
        include_dirs=[numpy.get_include()],
        package_dir={'': 'src'},
        test_suite='nose.collector',
        tests_require=['nose']
    )

    if len(sys.argv) >= 2 and ('--help' in sys.argv[1:] or
                                       sys.argv[1] in ('--help-commands', 'egg_info', '--version',
                                                       'clean')):
        # for these actions, a full build is not required.
        fullversion, git_revision = get_version_info()
        metadata['version'] = fullversion
    else:
        if len(sys.argv) >= 2 and sys.argv[1] == 'bdist_wheel':
            # bdist_wheel needs setuptools
            import setuptools

        cwd = os.path.abspath(os.path.dirname(__file__))
        if not os.path.exists(os.path.join(cwd, 'PKG-INFO')):
            # Generate Cython sources, unless building from source release
            generate_cython()

        # a pypore.version should exist
        metadata['version'] = _get_version_from_py()

    try:
        from setuptools import setup
    except ImportError:
        from distutils.core import setup

    packages = list(find_packages('src'))
    metadata['packages'] = packages

    ext_modules = list(find_extensions('src'))
    metadata['ext_modules'] = ext_modules

    setup(**metadata)


if __name__ == '__main__':
    setup_package()



import unittest
from PySide import QtGui
from PySide.QtGui import QCheckBox
from pyporegui.widgets.plot_tool_bar import PlotToolBar

# This class needs a QApplication instance
if QtGui.qApp is None:
    QtGui.QApplication([])


class TestPlotToolBar(unittest.TestCase):

    def test_plot_tool_bar_initial_state(self):
        toolbar = PlotToolBar()
        self.assertTrue(toolbar.is_decimate_checked())
        self.assertTrue(toolbar.is_plot_during_checked())
        widget_list = toolbar.get_widget_list()
        self.assertEqual(len(widget_list), 3)

    def test_plot_tool_bar_widget_list(self):
        toolbar = PlotToolBar()
        widget_list = toolbar.get_widget_list()
        self.assertEqual(len(widget_list), 3)

        # tests for immutability
        widget_list.append(QCheckBox())
        self.assertEqual(len(toolbar.get_widget_list()), 3)

        # tests adding widget
        checkbox = QCheckBox()
        toolbar.addWidget(checkbox)
        self.assertEqual(len(toolbar.get_widget_list()), 4)


import os

import numpy as np
from scipy import signal as sig
from pypore.filetypes import data_file
from pypore.i_o import get_reader_from_filename


def filter_file(filename, filter_frequency, out_sample_rate, output_filename=None):
    """
    Reads data from the filename file and uses a Butterworth low-pass filter with cutoff at filter_frequency. Outputs
    the filtered waveform to a new :py:class:`pypore.filetypes.data_file.DataFile`.

    :param StringType filename: Filename containing data to be filtered.
    :param DoubleType filter_frequency: Cutoff frequency for the low-pass Butterworth filter.
    :param DoubleType out_sample_rate: After the data is filtered, it can be resampled to roughly out_sample_rate. If \
            out_sample_rate <= 0, the data will not be resampled.
    :param StringType output_filename: (Optional) Filename for the filtered data. If not specified,
        for an example filename='test.mat', the default output_filename would be 'test_filtered.h5'
    :returns: StringType -- The output filename of the filtered data.

    Usage:

    >>> import pypore.file_converter as fc
    >>> fc.filter_file("filename", 1.e4, 1.e5, "output.h5") # filter at 10kHz, resample at 100kHz
    """
    reader = get_reader_from_filename(filename)

    data = reader.get_all_data()[0]

    sample_rate = reader.get_sample_rate()
    final_sample_rate = sample_rate
    n_points = len(data)

    if output_filename is None:
        output_filename = os.path.splitext(filename)[0] + '_filtered.h5'

    # wn is a fraction of the Nyquist frequency (half the sampling frequency).
    wn = filter_frequency / (0.5 * sample_rate)

    b, a = sig.butter(6, wn)

    filtered = sig.filtfilt(b, a, data)[:]

    # resample the data, if requested.
    if 0 < out_sample_rate < sample_rate:
        n_out = int(np.ceil(n_points * out_sample_rate / sample_rate))
        filtered = sig.resample(filtered, num=n_out)
        final_sample_rate = sample_rate * (1.0 * n_out / n_points)

    save_file = data_file.open_file(output_filename, mode='w', sample_rate=final_sample_rate, n_points=filtered.size)
    save_file.root.data[:] = filtered[:]

    save_file.flush()
    save_file.close()
    return output_filename




from PySide import QtGui


class PlotToolBar(QtGui.QToolBar):
    """
    A toolbar for plots, with a zoom button, check boxes for options.
    """

    def __init__(self, parent=None):
        super(PlotToolBar, self).__init__(parent)

        self.widget_list = []

        self.decimate_check_box = QtGui.QCheckBox()
        self.decimate_check_box.setChecked(True)
        self.decimate_check_box.setText('Decimate')
        self.addWidget(self.decimate_check_box)

        self.plot_during_check_box = QtGui.QCheckBox()
        self.plot_during_check_box.setChecked(True)
        self.plot_during_check_box.setText('Plot Events')
        self.plot_during_check_box.setToolTip('Select to have events plotted during event finding.')
        self.addWidget(self.plot_during_check_box)

        self.filter_data = QtGui.QCheckBox()
        self.filter_data.setChecked(True)
        self.filter_data.setText('Show filtered')
        self.filter_data.setToolTip('Select to have events plotted during event finding.')
        self.addWidget(self.filter_data)

    def is_decimate_checked(self):
        """
        :returns: boolean -- true if the toolbar's decimate checkbox is checked, false
                    otherwise.
        """
        return self.decimate_check_box.isChecked()

    def is_plot_during_checked(self):
        """
        :returns: boolean -- true if the toolbar's plot during checkbox is checked, false otherwise.
        """
        return self.plot_during_check_box.isChecked()

    def get_widget_list(self):
        """
        :returns: ListType<QtGui.QWidget> -- Returns a list of the widgets contained in the PlotToolBar.
        """
        return list(self.widget_list)

    def addWidget(self, widget, *args, **kwargs):
        self.widget_list.append(widget)
        return super(PlotToolBar, self).addWidget(widget, *args, **kwargs)


"""
Not a very useful class, basically just a list with less methods.
@author: `@parkin`_
"""


class _ThreadManager(object):
    def __init__(self, *args, **kargs):
        """
        """
        super(_ThreadManager, self).__init__()
        self.thread_pool = []

    def add_thread(self, thread):
        self.thread_pool.append(thread)

    def clean_threads(self):
        """
        Cancels all of the currently running threads.
        """
        for w in self.thread_pool:
            w.cancel()
            #             w.wait()
            self.thread_pool.remove(w)

    def remove_thread(self, thread):
        """
        Removes the thread from the threadPool.
        """
        self.thread_pool.remove(thread)





import unittest
from functools import wraps
import numpy as np

from PySide import QtGui

from pyporegui.graphicsItems.histogram_item import HistogramItem

# This test class needs a QApplication instance
if QtGui.qApp is None:
    QtGui.QApplication([])


def _rotation(func):
    """
    Decorates a test function, calls the test twice, once with rotate=True, second time with rotate=False.
    """
    @wraps(func)
    def wrap(self):
        print "rotate: True"
        func(self, rotate=True)
        print "rotate: False"
        func(self, rotate=False)

    return wrap


class TestHistogramItem(unittest.TestCase):
    def _assert_histogram_item_array_lengths(self, item, length):
        """
        Tests that all of the list members of the HistogramItem item have length number of entries.
        """
        self.assertEqual(len(item.listDataItems()), length)
        self.assertEqual(len(item.data_array), length)
        self.assertEqual(len(item.maximum_array), length)
        self.assertEqual(len(item.minimum_array), length)

    @_rotation
    def test_add_histogram(self, rotate):
        """
        Tests adding a histogram.
        """
        item = HistogramItem(rotate=rotate)

        # Assert no data items.
        self._assert_histogram_item_array_lengths(item, 0)

        data = np.random.random(100)

        item.add_histogram(data)

        data_items = item.listDataItems()
        self._assert_histogram_item_array_lengths(item, 1)

    @_rotation
    def test_add_2_histograms_same_data(self, rotate):
        """
        Tests that adding 2 histograms with the same data is ok.
        """
        item = HistogramItem(rotate=rotate)

        data = np.random.random(100)

        item.add_histogram(data)

        item.add_histogram(data)
        self._assert_histogram_item_array_lengths(item, 2)

    @_rotation
    def test_add_2nd_histogram_narrower_range(self, rotate):
        """
        Tests that we can add a 2nd histogram with narrower (max-min) data. This should not
        change the histograms max/min or the number of bins.
        """
        item = HistogramItem(rotate=rotate)

        # -50 < data[i] < 50
        data = np.random.random(100) * 100 - 50

        item.add_histogram(data)

        maximum = item.maximum
        minimum = item.minimum
        n_bins = item.n_bins

        # 0 < data[i] < 1
        data2 = np.random.random(100)
        item.add_histogram(data2)

        self._assert_histogram_item_array_lengths(item, 2)
        self.assertEqual(item.maximum, maximum)
        self.assertEqual(item.minimum, minimum)
        self.assertEqual(item.n_bins, n_bins)

    @_rotation
    def test_add_2nd_histogram_wider_range(self, rotate):
        """
        Tests that we can add a 2nd histogram with wider data. The max should increase, min
        should decrease, n_bins should stay same (default n_bins depends on length of data).
        """
        item = HistogramItem(rotate=rotate)

        # 0 < data[i] < 1
        data = np.random.random(100)
        # -50 < data[i] < 50
        data2 = np.random.random(100) * 100 - 50

        item.add_histogram(data)

        maximum = item.maximum
        minimum = item.minimum
        n_bins = item.n_bins

        item.add_histogram(data2)

        self._assert_histogram_item_array_lengths(item, 2)
        self.assertLess(item.minimum, minimum)
        self.assertGreater(item.maximum, maximum)
        self.assertEqual(item.n_bins, n_bins)

    @_rotation
    def test_add_2nd_histogram_more_points(self, rotate):
        """
        Tests that adding a 2nd histogram with more points increases the number of bins.
        """
        item = HistogramItem(rotate=rotate)

        data = np.random.random(100)

        item.add_histogram(data)
        n_bins = item.n_bins

        data2 = np.random.random(1000)

        item.add_histogram(data2)

        self._assert_histogram_item_array_lengths(item, 2)
        self.assertGreater(item.n_bins, n_bins)

    @_rotation
    def test_add_2nd_histogram_less_points(self, rotate):
        """
        Tests that adding a 2nd histogram with less points doesn't change number of bins.
        """
        item = HistogramItem(rotate=rotate)

        data = np.random.random(1000)

        item.add_histogram(data)
        n_bins = item.n_bins

        data2 = np.random.random(100)

        item.add_histogram(data2)

        self._assert_histogram_item_array_lengths(item, 2)
        self.assertEqual(item.n_bins, n_bins)

    @_rotation
    def test_remove_only_histogram(self, rotate):
        """
        Tests that removing the only item works.
        """
        item = HistogramItem(rotate=rotate)

        data = np.random.random(100)

        item.add_histogram(data)

        self._assert_histogram_item_array_lengths(item, 1)

        item.remove_item_at(0)

        self._assert_histogram_item_array_lengths(item, 0)

    @_rotation
    def test_remove_first_histogram(self, rotate):
        """
        Tests that removing the first histogram works.
        """
        item = HistogramItem(rotate=rotate)

        data = np.random.random(100)

        item.add_histogram(data)

        data2 = np.random.random(200)

        item.add_histogram(data2)

        data_item2 = item.listDataItems()[1]

        item.remove_item_at(0)

        self._assert_histogram_item_array_lengths(item, 1)

        data_item_left = item.listDataItems()[0]

        self.assertEqual(data_item2, data_item_left)



from pyqtgraph import PlotItem


class MyPlotItem(PlotItem):
    """
    A subclass of PlotItem that keeps track of regular items and event items separately.
    """
    def __init__(self, parent=None, title=None, name=None):
        super(MyPlotItem, self).__init__(parent=parent, title=title, name=name)
        self._myItemList = []
        self._myEventItemList = []

    def addItem(self, item, *args, **kwargs):
        super(MyPlotItem, self).addItem(item, *args, **kwargs)
        self._myItemList.append(item)

    def add_event_item(self, item, *args, **kwargs):
        super(MyPlotItem, self).addItem(item, *args, **kwargs)
        self._myEventItemList.append(item)

    def clear_event_items(self):
        for item in self._myEventItemList:
            self.removeItem(item)
        del self._myEventItemList[:]

    def clear(self):
        super(MyPlotItem, self).clear()
        del self._myEventItemList[:]
        del self._myItemList[:]


from functools import wraps
import inspect
import os


def _test_file_manager(directory):
    """
    Decorator. Creates a test filename based on the decorated function's name. Makes sure this file is removed before and after
    the decorated function is called. Calls the decorated function with an extra named parameter filename.

    :param directory: Directory in which to save the file.
    """

    def real_dec(function):
        @wraps(function)
        def wrap(*args, **kwargs):
            arg_spec = inspect.getargspec(function)
            filename = ""
            if 'self' in arg_spec.args:
                filename += args[0].__class__.__name__ + "_"
            filename += function.__name__ + '.h5'

            filename = os.path.join(directory, filename)

            if os.path.exists(filename):
                os.remove(filename)

            function(*args, filename=filename, **kwargs)

            if os.path.exists(filename):
                os.remove(filename)

        return wrap

    return real_dec



from PySide import QtGui, QtCore
import numpy as np
import pyqtgraph as pg
from pyqtgraph.widgets.LayoutWidget import LayoutWidget
from pypore.filetypes import event_database as eD
from pyporegui._thread_manager import _ThreadManager
from pyporegui.graphicsItems.my_plot_item import MyPlotItem
from pyporegui.file_items import FileListItem
from pyporegui.graphicsItems.path_item import PathItem

__all__ = ['EventViewingTab']


class EventViewingTab(_ThreadManager, QtGui.QSplitter):
    """
    A QtGui.Splitter that contains a file opener list on the left and plots on the right.
    """

    def __init__(self, parent=None):
        """
        :param PySide.QtGui.QMainWindow parent: Parent main window (optional).
        """
        super(EventViewingTab, self).__init__(parent)

        self._parent = parent

        self.open_directory_changed_callback = None

        options = self._create_event_viewer_options()
        plots = self._create_event_viewer_plot_widget()

        # Put everything in baseline_filter_parameter scroll area
        scroll_plots = QtGui.QScrollArea()
        scroll_plots.setWidgetResizable(True)

        scroll_plots.setWidget(plots)

        self.addWidget(options)
        self.addWidget(scroll_plots)

    def open_event_databases(self, file_names=None):
        """
        Adds the files to the list widget.

        :param ListType<StringType> file_names: (Optional) List of file names to be added to the list widget. If not
                                                included, then a QtGui.QFileDialog will be opened to select files.
        """
        if file_names is None:
            file_names = QtGui.QFileDialog.getOpenFileNames(self, 'Open event database', '.', '*.h5')[0]

        if len(file_names) > 0:
            self.eventview_list_widget.clear()
        else:
            return
        for w in file_names:
            item = FileListItem(w)
            self.eventview_list_widget.addItem(item)

    def _create_event_viewer_options(self):
        scroll_area = QtGui.QScrollArea()
        scroll_area.setWidgetResizable(True)

        # Create baseline_filter_parameter list for files want to analyze
        self.eventview_list_widget = QtGui.QListWidget()
        self.eventview_list_widget.itemDoubleClicked.connect(self._on_eventview_file_item_doubleclick)
        self.eventview_list_widget.setMaximumHeight(100)

        fixed_analysis_options = QtGui.QFormLayout()
        fixed_analysis_options.addRow('Event Databases:', self.eventview_list_widget)

        vbox_left = QtGui.QVBoxLayout()
        vbox_left.addLayout(fixed_analysis_options)

        vbox_left_widget = QtGui.QWidget()
        vbox_left_widget.setLayout(vbox_left)

        scroll_area.setWidget(vbox_left_widget)

        return scroll_area

    def _on_eventview_file_item_doubleclick(self, item):
        """
        """
        self.event_view_item = item

        h5file = eD.open_file(item.get_file_name())

        event_count = h5file.get_event_count()

        if h5file.is_debug():
            self.plot_debug(h5file)

        h5file.close()

        self.event_display_edit.setMaxLength(int(event_count / 10) + 1)
        self.event_display_edit.setValidator(QtGui.QIntValidator(1, event_count, self.event_display_edit))
        self.event_count_text.setText('/' + str(event_count))
        self.event_display_edit.setText('')
        self.event_display_edit.setText('1')

    def plot_debug(self, event_database):
        """
        Plots the data, baseline, and thresholds of the debug group in the event_database, if they exist,
        in the main plot.

        :param event_database: An already open\
                :class:`EventDatabase <pypore.filetypes.event_database.EventDatabase>`.
        """
        if not event_database.is_debug():
            return

        self.eventview_plotwid.clear()

        sample_rate = event_database.get_sample_rate()

        # TODO remove the step_size.
        step_size = 1000

        data = event_database.root.debug.data[0][::step_size]

        data_size = data.size
        times = np.linspace(0, data_size *1.0/sample_rate, data_size)
        item = PathItem(times, data)
        item.setPen(pg.mkPen('w'))
        self.eventview_plotwid.addItem(item)

        baseline = event_database.root.debug.baseline[0][::step_size]
        item = PathItem(times, baseline)
        item.setPen(pg.mkPen('y'))
        self.eventview_plotwid.addItem(item)

        threshold_p = event_database.root.debug.threshold_positive[0][::step_size]
        item = PathItem(times, threshold_p)
        item.setPen(pg.mkPen('g'))
        self.eventview_plotwid.addItem(item)

        threshold_n = event_database.root.debug.threshold_negative[0][::step_size]
        item = PathItem(times, threshold_n)
        item.setPen(pg.mkPen('g'))
        self.eventview_plotwid.addItem(item)

    def plot_single_events(self, event):
        """
        Plots the event on the plot with
        """
        h5file = eD.open_file(self.event_view_item.get_file_name(), mode='r')

        event_count = h5file.get_event_count()

        for i in xrange(3):
            for j in xrange(3):
                pos = 3 * i + j
                if pos + event >= event_count or pos + event < 0:
                    self.eventviewer_plots[pos].clear()
                    self.eventviewer_plots[pos].setTitle('')
                else:
                    self.plot_single_event(h5file, event + pos, self.eventviewer_plots[pos])
                    self.eventviewer_plots[pos].setTitle('Event ' + str(event + pos + 1))

        h5file.close()

    def plot_single_event(self, h5file, position, plot):
        sample_rate = h5file.get_sample_rate()
        row = h5file.get_event_row(position)
        event_length = row['event_length']
        raw_points_per_side = row['raw_points_per_side']

        raw_data = h5file.get_raw_data_at(position)

        n = len(raw_data)

        times = np.linspace(0.0, 1.0 * n / sample_rate, n)

        plot.clear()
        plot.plot(times, raw_data)
        # plot the event points in yellow
        plot.plot(times[raw_points_per_side:raw_points_per_side + event_length],
                  raw_data[raw_points_per_side:raw_points_per_side + event_length], pen='y')

        # Plot the cusum levels
        n_levels = row['n_levels']
        baseline = row['baseline']
        # left, start-1, start,
        levels = h5file.get_levels_at(position)
        indices = h5file.get_level_lengths_at(position)

        level_times = np.zeros(2 * n_levels + 4)
        level_values = np.zeros(2 * n_levels + 4)

        level_times[1] = 1.0 * (raw_points_per_side - 1) / sample_rate
        level_values[0] = level_values[1] = baseline
        i = 0
        length = 0
        for i in xrange(n_levels):
            level_times[2 * i + 2] = times[raw_points_per_side] + 1.0 * length / sample_rate
            level_values[2 * i + 2] = levels[i]
            level_times[2 * i + 3] = times[raw_points_per_side] + 1.0 * (length + indices[i]) / sample_rate
            level_values[2 * i + 3] = levels[i]
            length += indices[i]
        i += 1
        level_times[2 * i + 2] = times[raw_points_per_side + event_length]
        level_times[2 * i + 3] = times[n - 1]
        level_values[2 * i + 2] = level_values[2 * i + 3] = baseline

        plot.plot(level_times, level_values, pen='g')

    def previous_clicked(self):
        self.move_event_display_by(-1 * len(self.eventviewer_plots))

    def next_clicked(self):
        self.move_event_display_by(len(self.eventviewer_plots))

    def move_event_display_by(self, count):
        """
        Changes the event displayed on the event display plot to
        current value + count
        """
        h5_event_count = 0
        try:
            h5file = eD.open_file(self.event_view_item.get_file_name())
            h5_event_count = h5file.get_event_count()
            h5file.close()
        except:
            return
        try:
            event_count = int(self.event_display_edit.text())
            if 0 < event_count + count <= h5_event_count:
                self.event_display_edit.setText(str(event_count + count))
        except ValueError:
            # if we can't parse the event display text but there are events,
            # just set to zero
            if h5_event_count > 0:
                self.event_display_edit.setText('1')

    def _event_display_edit_on_change(self, text):
        if len(text) < 1:
            return
        position = int(self.event_display_edit.text())
        self.plot_single_events(position - 1)
        return

    def _create_event_viewer_plot_widget(self):
        wig = pg.GraphicsLayoutWidget()
        wig2 = pg.GraphicsLayoutWidget()

        # Main plot
        self.eventview_plotwid = MyPlotItem(title='Current Trace', name='Plot')
        wig.addItem(self.eventview_plotwid)
        self.eventview_plotwid.enableAutoRange('xy', False)

        wig.nextRow()
        # Create Qwt plot for concatenated events
        self.plot_concatevents = wig.addPlot(title='Concatenated Events', name='Concat')

        self.eventviewer_plots = []

        # Now add 9 plots to view events in
        for i in xrange(3):
            wig2.nextRow()
            for j in xrange(3):
                plot = wig2.addPlot(title='Event ' + str(i * 3 + j), name='Single' + str(i * 3 + j))
                self.eventviewer_plots.append(plot)

                # Tool bar for main plot.  Contains zoom button and different checkboxes
                #         self.plotToolBar = PlotToolBar(self)
                #         self.addToolBar(self.plotToolBar)

        event_select_toolbar = QtGui.QToolBar(self)
        if self._parent is not None:
            self._parent.addToolBar(event_select_toolbar)

        button_previous = QtGui.QPushButton(event_select_toolbar)
        button_previous.setText("Previous")
        button_previous.clicked.connect(self.previous_clicked)
        event_select_toolbar.addWidget(button_previous)

        self.event_display_edit = QtGui.QLineEdit()
        self.event_display_edit.setText('0')
        self.event_display_edit.setMaxLength(1)
        self.event_display_edit.setValidator(QtGui.QIntValidator(0, 0, self.event_display_edit))
        self.event_display_edit.textChanged.connect(self._event_display_edit_on_change)
        event_select_toolbar.addWidget(self.event_display_edit)

        self.event_count_text = QtGui.QLabel()
        self.event_count_text.setText('/' + str(0))
        event_select_toolbar.addWidget(self.event_count_text)

        button_previous = QtGui.QPushButton(event_select_toolbar)
        button_previous.setText("Next")
        button_previous.clicked.connect(self.next_clicked)
        event_select_toolbar.addWidget(button_previous)

        frame = QtGui.QSplitter()
        frame.setOrientation(QtCore.Qt.Vertical)
        frame.addWidget(wig)
        frame.addWidget(wig2)

        event_finder_plots_layout = LayoutWidget()
        # event_finder_plots_layout.addWidget(self.plotToolBar, row=1, col=0, colspan=3)
        event_finder_plots_layout.addWidget(frame, row=2, col=0, colspan=3)
        event_finder_plots_layout.addWidget(event_select_toolbar, row=5, col=0, colspan=3)

        return event_finder_plots_layout





from PySide import QtGui, QtCore
import pyqtgraph as pg


class PathItem(QtGui.QGraphicsPathItem):
    def __init__(self, x, y, conn='all'):
        xr = x.min(), x.max()
        yr = y.min(), y.max()
        self._bounds = QtCore.QRectF(xr[0], yr[0], xr[1] - xr[0], yr[1] - yr[0])
        self.path = pg.arrayToQPath(x, y, conn)
        QtGui.QGraphicsPathItem.__init__(self, self.path)

    def boundingRect(self):
        return self._bounds




"""
Created on Sep 13, 2013

@author: `@parkin`_
"""
import unittest
import os

import numpy as np
import numpy.testing as npt

import pypore.filetypes.event_database as eD


class TestEventDatabase(unittest.TestCase):
    def setUp(self):
        self.filename = 'testEventDatabase_938247283278128.h5'
        self.max_event_length = 100
        self.database = eD.open_file(self.filename, mode='w', maxEventLength=self.max_event_length)

    def tearDown(self):
        self.database.close()
        os.remove(self.filename)

    def _test_initial_root(self, file_h=None):
        """
        Check that only the root group and event group exist
        """
        if file_h is None:
            file_h = self.database

        names = [x._v_name for x in file_h.walkGroups()]
        self.assertIn('/', names, 'file missing root group')
        self.assertIn('events', names, "file missing event group.")

    def _test_empty_events_group(self, database=None, events_group=None):
        """
        Checks that the events group has the correct (and empty) tables/arrays
        """
        if database is None:
            database = self.database
        if events_group is None:
            events_group = database.root.events

        # Check the events group has the correct nodes
        # Should be events, eventTable, eventData, raw_data
        names = []
        for node in database.walkNodes(events_group):
            names.append(node._v_name)
        names = [x._v_name for x in database.walkNodes(events_group)]
        names_should_be = ['events', 'eventTable', 'raw_data', 'levels', 'level_lengths']
        self.assertEqual(len(names), len(names_should_be))
        self.assertEqual(sorted(names), sorted(names_should_be))  # i don't care what order these lists are

        # Assert table, vlarrays are empty
        self.assertEqual(events_group.eventTable.nrows, 0)
        self.assertEqual(events_group.raw_data.nrows, 0)
        self.assertEqual(events_group.levels.nrows, 0)
        self.assertEqual(events_group.level_lengths.nrows, 0)

        # Check the eventTable columns are correct and in correct order
        column_names = ['array_row', 'event_start', 'event_length', 'n_levels', 'raw_points_per_side', 'baseline',
                        'current_blockage', 'area']
        self.assertEqual(events_group.eventTable.colnames, column_names)

    def test_clean_database(self):
        """
        Tests that cleanDatabase removing eventsTable, raw_data, levels, and levelData
        and initializing with empty table/matrices.
        """
        # put stuff in raw_data
        raw_data = np.ones((1, self.database.max_event_length))
        self.database.append_event(1, 2, 3, 4, 5, 6, 7, 8, raw_data, raw_data, raw_data)

        # clean the database
        self.database.clean_database()

        # check that the database has all the empty table/matrices
        self._test_empty_events_group(events_group=self.database.root.events)

    def test_append_event(self):
        """
        Tests appendEvent
        
        array_row, event_start, event_length, n_levels, raw_points_per_side,\
                    baseline, current_blockage, area, raw_data = None, levels = None, level_lengths = None
        """
        table = self.database.get_event_table()

        # add an event
        self.database.append_event(1, 2, 3, 4, 5, 6, 7, 8)
        self.database.flush()

        # Check eventTable for one correct row.
        self.assertEqual(self.database.get_event_table().nrows, 1)
        self.assertEqual(table[0]['array_row'], 1)
        self.assertEqual(table[0]['event_start'], 2)
        self.assertEqual(table[0]['event_length'], 3)
        self.assertEqual(table[0]['n_levels'], 4)
        self.assertEqual(table[0]['raw_points_per_side'], 5)
        self.assertEqual(table[0]['baseline'], 6)
        self.assertEqual(table[0]['current_blockage'], 7)
        self.assertEqual(table[0]['area'], 8)

        # Check for nothing in raw_data, levels, level_lengths
        self.assertEqual(self.database.root.events.raw_data.nrows, 0)
        self.assertEqual(self.database.root.events.levels.nrows, 0)
        self.assertEqual(self.database.root.events.level_lengths.nrows, 0)

        # add another event, this time with raw_data
        raw_data = np.ones((1, self.database.max_event_length))
        self.database.append_event(10, 20, 30, 40, 50, 60, 70, 80, raw_data)
        self.database.flush()

        # Check eventTable for two correct rows.
        self.assertEqual(self.database.get_event_table().nrows, 2)
        self.assertEqual(table[1]['array_row'], 10)
        self.assertEqual(table[1]['event_start'], 20)
        self.assertEqual(table[1]['event_length'], 30)
        self.assertEqual(table[1]['n_levels'], 40)
        self.assertEqual(table[1]['raw_points_per_side'], 50)
        self.assertEqual(table[1]['baseline'], 60)
        self.assertEqual(table[1]['current_blockage'], 70)
        self.assertEqual(table[1]['area'], 80)

        # Check for 1 row in raw_data and nothing in levels, level_lengths
        self.assertEqual(self.database.root.events.raw_data.nrows, 1)
        self.assertEqual(self.database.root.events.levels.nrows, 0)
        self.assertEqual(self.database.root.events.level_lengths.nrows, 0)
        npt.assert_array_equal(self.database.root.events.raw_data[:], raw_data)

    def test_delete_event(self):
        """
        Tests deleting single events from eventTable.
        """
        table = self.database.get_event_table()

        # Test deleting all events in database does nothing
        self.database.append_event(1, 2, 3, 4, 5, 6, 7, 8)
        self.database.remove_event(0)
        self.assertEqual(self.database.get_event_count(), 1)

        # Test deleting out of range does nothing
        self.database.remove_event(-1)
        self.assertEqual(self.database.get_event_count(), 1)
        self.database.remove_event(1)
        self.assertEqual(self.database.get_event_count(), 1)
        self.database.remove_event(10)
        self.assertEqual(self.database.get_event_count(), 1)

        # Test deleting first event in database
        for i in xrange(1, 10):
            self.database.append_event(i + 1, i + 2, i + 3, i + 4, i + 5, i + 6, i + 7, i + 8)
        self.assertEqual(self.database.get_event_count(), 10)
        self.assertEqual(table[0]['array_row'], 1)
        self.database.remove_event(0)
        self.assertEqual(self.database.get_event_count(), 9)
        self.assertEqual(table[0]['array_row'], 2)

        # Test deleting last event in database
        self.assertEqual(table[8]['array_row'], 10)
        self.database.remove_event(8)
        self.assertEqual(self.database.get_event_count(), 8)
        self.assertEqual(table[7]['array_row'], 9)

        # Test deleting second to last row
        self.assertEqual(table[6]['array_row'], 8)
        self.database.remove_event(6)
        self.assertEqual(self.database.get_event_count(), 7)
        self.assertEqual(table[6]['array_row'], 9)
        self.assertEqual(table[5]['array_row'], 7)

    def test_delete_events(self):
        """
        Test removing ranges of events from event table.
        """
        table = self.database.get_event_table()

        # Test deleting all events in database does nothing
        self.database.append_event(1, 2, 3, 4, 5, 6, 7, 8)
        self.database.remove_events(0, 1)
        self.assertEqual(self.database.get_event_count(), 1)

        # Add to total of 10 events
        for i in xrange(1, 10):
            self.database.append_event(i + 1, i + 2, i + 3, i + 4, i + 5, i + 6, i + 7, i + 8)

        # Test deleting all events does nothing
        self.database.remove_events(0, 10)
        self.assertEqual(self.database.get_event_count(), 10)

        # Test for i out of range
        self.database.remove_events(-1, 5)
        self.assertEqual(self.database.get_event_count(), 10)
        self.database.remove_events(-9999, 5)
        self.assertEqual(self.database.get_event_count(), 10)
        self.database.remove_events(5, 5)
        self.assertEqual(self.database.get_event_count(), 10)
        self.database.remove_events(6, 5)
        self.assertEqual(self.database.get_event_count(), 10)
        self.database.remove_events(50, 5)
        self.assertEqual(self.database.get_event_count(), 10)
        self.database.remove_events(50, 51)
        self.assertEqual(self.database.get_event_count(), 10)

        # Test for j out of range
        self.database.remove_events(5, 5)
        self.assertEqual(self.database.get_event_count(), 10)
        self.database.remove_events(5, 4)
        self.assertEqual(self.database.get_event_count(), 10)
        self.database.remove_events(5, -1)
        self.assertEqual(self.database.get_event_count(), 10)
        self.database.remove_events(5, -100)
        self.assertEqual(self.database.get_event_count(), 10)
        self.database.remove_events(5, 11)
        self.assertEqual(self.database.get_event_count(), 10)
        self.database.remove_events(5, 100)
        self.assertEqual(self.database.get_event_count(), 10)

        # Test removing first event
        self.database.remove_events(0, 1)
        self.assertEqual(self.database.get_event_count(), 9)
        self.assertEqual(table[0]['array_row'], 2)

        # Test removing final event
        self.database.remove_events(8, 9)
        self.assertEqual(self.database.get_event_count(), 8)
        self.assertEqual(table[7]['array_row'], 9)

        # Test removing range
        self.database.remove_events(1, 7)
        self.assertEqual(self.database.get_event_count(), 2)
        self.assertEqual(table[0]['array_row'], 2)
        self.assertEqual(table[1]['array_row'], 9)

    def test_initialize_events_database(self):
        self._test_initial_root(self.database)

        self._test_empty_events_group()

        # Check is in write mode
        self.database.root.events.raw_data.append(np.zeros((1, 100)))
        self.database.flush()

    def test_initialize_events_database_with_debug(self):
        filename = 'test_initialize_events_database_with_debug.h5'
        if os.path.exists(filename):
            os.remove(filename)
        n_points = 100
        n_channels = 2
        database = eD.open_file(filename, mode='w', debug=True, n_points=n_points, n_channels=n_channels)

        self._test_initial_root(database)

        self._test_empty_events_group(events_group=database.root.events)

        # Make sure is debug
        self.assertTrue(database.is_debug())

        # Make sure the debug group is there.
        names = [x._v_name for x in database.walkGroups()]
        self.assertIn('debug', names, 'No debug group.')

        debug_group = database.root.debug

        # Make sure the debug group
        debug_group_names = [x._v_name for x in database.walkNodes(debug_group)]
        print debug_group_names
        self.assertIn('data', debug_group_names, 'No data matrix.')
        self.assertIn('baseline', debug_group_names, 'No baseline matrix.')
        self.assertIn('threshold_positive', debug_group_names, 'No positive threshold matrix.')
        self.assertIn('threshold_negative', debug_group_names, 'No negative threshold matrix.')

        # Make sure arrays are correct dimensions
        self.assertEqual(debug_group.data.shape, (n_channels, n_points))
        self.assertEqual(debug_group.baseline.shape, (n_channels, n_points))
        self.assertEqual(debug_group.threshold_positive.shape, (n_channels, n_points))
        self.assertEqual(debug_group.threshold_negative.shape, (n_channels, n_points))

        # Check is in write mode
        database.root.events.raw_data.append(np.zeros((1, 100)))
        database.flush()

        database.close()
        os.remove(filename)

    def test_open_existing_empty_database(self):
        """
        Tests opening an existing h5 file with an empty 
        EventsDatabase structure.
        """
        self.database.close()

        self.database = eD.open_file(self.filename)

        self._test_initial_root(self.database)

        self._test_empty_events_group()

    def test_open_existing_full_database(self):
        """
        Tests opening an existing h5 file with events in
        the EventDatabase structure.
        """
        # Add to the raw_data matrix
        raw_data = np.zeros((2, self.max_event_length))
        raw_data[1][:] += 1
        self.database.append_raw_data(raw_data)

        # Add to the levels matrix
        levels = raw_data + 1
        self.database.append_levels(levels)

        # Add to the level_lengths matrix
        level_lengths = raw_data + 2
        self.database.append_level_lengths(level_lengths)

        # Add to the event table
        event_row = self.database.get_event_table_row()
        event_row['array_row'] = 1
        event_row['event_start'] = 2
        event_row['event_length'] = 3
        event_row['n_levels'] = 4
        event_row['raw_points_per_side'] = 5
        event_row['baseline'] = 6
        event_row['current_blockage'] = 7
        event_row['area'] = 8
        event_row.append()

        # Close the file
        self.database.flush()
        self.database.close()

        # Open the existing file
        self.database = eD.open_file(self.filename, mode='r')

        # Check the raw_data matrix
        npt.assert_array_equal(raw_data, self.database.root.events.raw_data[:])

        # Check the levels matrix
        npt.assert_array_equal(levels, self.database.root.events.levels[:])

        # Check the level_lengths matrix
        npt.assert_array_equal(level_lengths, self.database.root.events.level_lengths[:])

        # Check the eventTable
        row = self.database.root.events.eventTable[0]
        self.assertEqual(row['array_row'], 1)
        self.assertEqual(row['area'], 8)

    def test_initialize_database_with_existing_nodes(self):
        """
        Test that calling initialize_c database with existing nodes does not
        throw an error (ie existing nodes ignored).
        """
        self.assertIn('eventTable', self.database.root.events)
        self.assertIn('raw_data', self.database.root.events)

        raw = np.ones((1, self.max_event_length))
        self.database.append_raw_data(raw)

        # tests to see if error thrown
        self.database.initialize_database()

        # check that raw_data is still the same
        npt.assert_array_equal(raw, self.database.root.events.raw_data[:])

    def test_get_events_group(self):
        """
        Test the getter for /events
        """
        events_group = self.database.get_events_group()
        self._test_empty_events_group(events_group=events_group)

        self.assertIn('raw_data', events_group)
        self.assertIn('levels', events_group)
        self.assertIn('level_lengths', events_group)

    def test_get_event_count(self):
        """
        Tests that getEventCount returns the correct value, even
        after events are added/deleted.
        """
        self.assertEqual(self.database.get_event_count(), 0)

        for i in xrange(100):
            self.database.append_event(i + 1, i + 2, i + 3, i + 4, i + 5, i + 6, i + 7, i + 8)
            self.assertEqual(self.database.get_event_count(), i + 1)

        for i in xrange(50):
            self.database.remove_event(0)
            self.assertEqual(self.database.get_event_count(), 99 - i)

    def test_get_event_data_at(self):
        """
        Tests getting only the event points from raw_data
        """
        raw = np.linspace(0.0, 100.0, self.max_event_length).reshape((1, 100))

        # event_length=3, raw_points_per_side=5
        self.database.append_event(0, 2, 3, 4, 5, 6, 7, 8, raw)

        row = self.database.get_event_row(0)
        event_length = row['event_length']
        raw_points_per_side = row['raw_points_per_side']

        self.assertEqual(event_length, 3)
        self.assertEqual(raw_points_per_side, 5)
        npt.assert_array_equal(self.database.get_event_data_at(0),
                               raw[0][raw_points_per_side:event_length + raw_points_per_side])

    def test_get_event_row(self):
        """
        Test that getEventRow returns correct row.
        """
        for i in xrange(10):
            self.database.append_event(i + 1, i + 2, i + 3, i + 4, i + 5, i + 6, i + 7, i + 8)

        row = self.database.get_event_row(1)
        self.assertEqual(row['array_row'], 2)

    def test_get_level_lengths_at(self):
        """
        Tests that getLevelLengthsAt returns a numpy matrix of the 
        level_lengths corresponding to the event
        in row 'i' of eventTable.
        """
        level_lengths = np.array(range(self.max_event_length)).reshape((1, self.max_event_length))

        self.database.append_event(0, 2, 3, 4, 5, 6, 7, 8,
                                   level_lengths=level_lengths)  # event_length=3, raw_points_per_side=5

        row = self.database.get_event_row(0)
        n_levels = row['n_levels']

        self.assertEqual(n_levels, 4)
        npt.assert_array_equal(self.database.get_level_lengths_at(0), level_lengths[0][:n_levels])

    def test_get_levels_at(self):
        """
        Tests that getLevelsAt returns a numpy matrix of the levels corresponding to the event
        in row 'i' of eventTable.
        """
        levels = np.linspace(0.0, 100.0, self.max_event_length).reshape((1, self.max_event_length))

        self.database.append_event(0, 2, 3, 4, 5, 6, 7, 8, levels=levels)  # event_length=3, raw_points_per_side=5

        row = self.database.get_event_row(0)
        n_levels = row['n_levels']

        self.assertEqual(n_levels, 4)
        npt.assert_array_equal(self.database.get_levels_at(0), levels[0][:n_levels])

    def test_get_raw_data_at(self):
        """
        Tests that get_raw_data_at returns correctly sized(event_length + 2*raw_points_per_side)
        numpy array with correct data.
        """
        raw = np.linspace(0.0, 100.0, self.max_event_length).reshape((1, self.max_event_length))

        self.database.append_event(0, 2, 3, 4, 5, 6, 7, 8, raw)  # event_length=3, raw_points_per_side=5

        row = self.database.get_event_row(0)
        event_length = row['event_length']
        raw_points_per_side = row['raw_points_per_side']

        self.assertEqual(event_length, 3)
        self.assertEqual(raw_points_per_side, 5)
        npt.assert_array_equal(self.database.get_raw_data_at(0), raw[0][:event_length + 2 * raw_points_per_side])

    def test_get_sample_rate(self):
        """
        Test that get_sample_rate returns root.events.eventTable.attrs.sample_rate
        """
        rate = 13928.99
        self.database.root.events.eventTable.attrs.sample_rate = rate

        self.assertEqual(self.database.get_sample_rate(), rate)

    def test_to_csv_default_columns(self):
        import csv

        event_start = 2
        event_length = 3
        n_levels = 4
        baseline = 10.0
        current_blockage = 9.0

        event_start_2 = 19
        event_length_2 = 20
        n_levels_2 = 8
        baseline_2 = -12.
        current_blockage_2 = 99.

        self.database.append_event(array_row=0, event_start=event_start, event_length=event_length, n_levels=n_levels,
                                   raw_points_per_side=5,
                                   baseline=baseline,
                                   current_blockage=current_blockage, area=8)
        self.database.append_event(array_row=1, event_start=event_start_2, event_length=event_length_2,
                                   n_levels=n_levels_2,
                                   raw_points_per_side=5,
                                   baseline=baseline_2,
                                   current_blockage=current_blockage_2, area=16)
        # raw_points_per_side=5
        # def append_event(self, array_row, event_start, event_length, n_levels, raw_points_per_side, baseline,
        # current_blockage, area, raw_data=None, levels=None, level_lengths=None):

        sample_rate = 10.e4
        self.database.get_event_table().attrs.sample_rate = sample_rate
        self.database.flush()

        output_filename = "blahblah.csv"
        if os.path.exists(output_filename):
            os.remove(output_filename)

        self.database.to_csv(output_filename)

        self.assertTrue(os.path.exists(output_filename))

        ifile = open(output_filename, 'rb')
        reader = csv.reader(ifile)

        # get header
        headers = next(reader, None)
        headers_should_be = ['Start Time', 'Dwell Time', 'Blockage', 'Baseline', 'Inter Event Time', 'Number of Levels']

        for i, header in enumerate(headers):
            self.assertEqual(header, headers_should_be[i])

        row_count = 0
        row_should_be = [event_start / sample_rate, event_length / sample_rate, current_blockage, baseline,
                         event_start / sample_rate,
                         n_levels]
        row_should_be_2 = [event_start_2 / sample_rate, event_length_2 / sample_rate, current_blockage_2, baseline_2,
                           (event_start_2 - event_start) / sample_rate,
                           n_levels_2]
        row = next(reader, None)
        self.assertEqual(len(row), len(row_should_be))
        for i, el in enumerate(row):
            self.assertAlmostEqual(float(el), row_should_be[i])
        row_count += 1

        row_2 = next(reader, None)
        self.assertEqual(len(row_2), len(row_should_be_2))
        for i, el in enumerate(row_2):
            self.assertAlmostEqual(float(el), row_should_be_2[i])
        row_count += 1

        for row_i in reader:
            row_count += 1

        self.assertEqual(row_count, 2)

        os.remove(output_filename)


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


"""
Created on Aug 27, 2013

@author: `@parkin`_
"""
import unittest
from pyporegui.graphicsItems.path_item import PathItem
import numpy as np
from PySide import QtGui

# This test class needs a QApplication instance
if QtGui.qApp is None:
    QtGui.QApplication([])


class TestPathItem(unittest.TestCase):
    def test_bounding_rect(self):
        x = np.random.uniform(-10, 10, size=10)
        y = np.random.uniform(-10, 10, size=10)

        x_max = x.max()
        x_min = x.min()
        y_max = y.max()
        y_min = y.min()

        item = PathItem(x, y)
        rect = item.boundingRect()
        rect_left = rect.left()
        self.assertAlmostEqual(rect_left, x_min, 7)
        rect_top = rect.top()
        self.assertAlmostEqual(rect_top, y_min, 7)
        wid = rect.width()
        hei = rect.height()
        self.assertAlmostEqual(wid, x_max - x_min, 7)
        self.assertAlmostEqual(hei, y_max - y_min)

    def test_bounding_rect_small_values(self):
        x = np.random.uniform(-1.e-9, 1.e-9, size=10)
        y = np.random.uniform(-1.e-9, 1.e-9, size=10)

        x_max = x.max()
        x_min = x.min()
        y_max = y.max()
        y_min = y.min()

        item = PathItem(x, y)
        rect = item.boundingRect()
        rect_left = rect.left()
        self.assertAlmostEqual(rect_left, x_min, 12)
        rect_top = rect.top()
        self.assertAlmostEqual(rect_top, y_min, 12)
        wid = rect.width()
        hei = rect.height()
        self.assertAlmostEqual(wid, x_max - x_min, 12)
        self.assertAlmostEqual(hei, y_max - y_min)


if __name__ == "__main__":
    unittest.main()
